; Draconian
; Last Update:  July 13, 2017
; Progam by:    Darrell Spice, Jr.

    PROCESSOR 6502
    
TIA_BASE_READ_ADDRESS = $30    
    INCLUDE "vcs.h"
    INCLUDE "macro.h"
;    INCLUDE "cdf_v1.h"
    INCLUDE "cdf_v1_1.h"

;===============================================================================
; Constants
;-------------------------------------------------------------------------------
;===============================================================================
;NO_ILLEGAL_OPCODES = 0

; Used to specify which ARM routine to run 
RUN_INIT    = 0     ; Initialize()
RUN_MENU_VB = 1     ; MenuVerticalBlank()
RUN_MENU_OS = 2     ; MenuOverScan()
RUN_GAME_VB = 3     ; GameVerticalBlank()
RUN_GAME_OS = 4     ; GameOverScan()
RUN_SPLASH_VB = 5   ; SplashVerticalBlank()
RUN_SPLASH_OS = 6   ; SplashOverScan()
RUN_INIT_LEVEL = 7  ; InitLevel()

;----------------------------------------
; Set this to adjust scanlines used for gameplay game area.  
; The +5 is for 5 scanlines to reposition all objects,
; this will be consistantly 5 scanlines vs using PosObject which
; uses an additional scanline for each object that's on the
; right side of the screen
;----------------------------------------
REPOSITION_SIZE = 5
DATASTREAM_SIZE = 160
RADAR_HEIGHT    = 20    ; radar is 16 pixels up/down with 2 extra at top/bottom for spacing
MAX_STARS       = 128   ; needs to be a power of 2

; these used to be calculated.  Now hardcoded so images can be run through DOOD
StationHorizontalHeight = $41
StationVerticalHeight   = $39
Ship0Height             = $10


;----------------------------------------
; NTSC color names.  PAL and SECAM color values are created on the fly in
; the custom C code.
;----------------------------------------
BLACK           = $00
WHITE           = $0E
GREY            = $00
YELLOW          = $10
ORANGE          = $20
BROWN           = $30
RED             = $40
PURPLE          = $50
VIOLET          = $60
INDIGO          = $70
BLUE            = $80
BLUE2           = $90
TURQUOISE       = $A0
CYAN            = $B0
GREEN           = $C0
YELLOW_GREEN    = $D0
OCHRE_GREEN     = $E0
OCHRE           = $F0
SCORE_COLOR     = $0A

; datastream usage for menu
; DS_LOGO_A           = DS0DATA
; DS_LOGO_B           = DS1DATA
; DS_LOGO_C           = DS2DATA
; DS_LOGO_D           = DS3DATA
; DS_LOGO_E           = DS4DATA
; DS_LOGO_F           = DS5DATA
; DS_LOGO_G           = DS6DATA
; DS_LOGO_H           = DS7DATA
; DS_LOGO_I           = DS8DATA
; DS_LOGO_J           = DS9DATA
; DS_LOGO_K           = DS10DATA
; DS_LOGO_L           = DS11DATA
; DS_PF2              = DS12DATA
; DS_LOGO_COLORS      = DS13DATA
DS_MENU_CONTROL     = DS14DATA
DS_MENU_A           = DS15DATA
DS_MENU_COLOR       = DS16DATA

; datastream usage for game screen
DS_GRP0             = DS0DATA
DS_GRP1             = DS1DATA
DS_HMP0             = DS2DATA
DS_HMP1             = DS3DATA
DS_MISSILE0         = DS4DATA   ; HMM0 and ENAM0
DS_MISSILE1         = DS5DATA   ; HMM1 and ENAM1
DS_BALL             = DS6DATA   ; HMBL and ENABL
DS_COLOR            = DS7DATA   ; color change for players and ball only
DS_SIZE             = DS8DATA   ; size change for all objects
DS_RADAR_CONTROL    = DS9DATA
DS_RADAR_GRP0       = DS10DATA
DS_RADAR_GRP1       = DS11DATA
DS_FORMATION_GRP0   = DS12DATA
DS_FORMATION_GRP1   = DS13DATA
DS_PF0L             = DS14DATA
DS_PF1L             = DS15DATA
DS_PF2L             = DS16DATA
DS_PF0R             = DS17DATA
DS_PF1R             = DS18DATA
DS_PF2R             = DS19DATA
DS_STATION_COLOR    = DS20DATA

;
; NOTE due to use of VDELBL 

; Constants for Main Menu
MM_TITLE_GAP    = 10
MM_OPTION_GAP   = 2
MM_START_GAP    = 9
MM_SCORE_GAP    = 11
MM_LOGO_GAP     = 11
MM_END          = $80

 
HMOVE_L7          =  $70
HMOVE_L6          =  $60
HMOVE_L5          =  $50
HMOVE_L4          =  $40
HMOVE_L3          =  $30
HMOVE_L2          =  $20
HMOVE_L1          =  $10
HMOVE_0           =  $00
HMOVE_R1          =  $F0
HMOVE_R2          =  $E0
HMOVE_R3          =  $D0
HMOVE_R4          =  $C0
HMOVE_R5          =  $B0
HMOVE_R6          =  $A0
HMOVE_R7          =  $90
HMOVE_R8          =  $80

    MAC DIGITAL_AUDIO
        lda #AMPLITUDE
        sta AUDV0
;        dec Sleep5
    ENDM

;===============================================================================
; Zero Page RAM  
;-------------------------------------------------------------------------------
; ZP RAM can only be seen by the 6507.  Any values that the ARM needs to see
; must be stored in the 4K Display Data bank
;===============================================================================
    SEG.U VARS
    ORG $80
NextKernel:     ds 2            
TimeLeftVB:     ds 1    ; NOTE this used during init to pass 2600/7800
TimeLeftOS:     ds 1    
LoopCounter:    ds 1
Mode:           ds 1    ; 0 = menu, 1 = game, 128 = splash
Frame:          ds 1
FormationColor: ds 1
Sleep5:         ds 1    ; dec/inc Sleep5 for 5 cycle sleep in 2 bytes

    ; Display Remaining RAM
    echo "---- RIOT RAM", ($100 - *), "bytes free" 

;===============================================================================
; Display Data RAM  
;-------------------------------------------------------------------------------
; 4K of RAM
;===============================================================================
    SEG.U DISPLAYDATA
    ORG $00

; data that's the same for menu and game
    
; This section is used to pass values between the 6507 and the ARM       
; ---- START OF A DATASTREAM ----    
; from 6507 to ARM
ARMfunc:    ds 1    ; which ARM function to run    
ARMswcha:   ds 1    ; controller state
ARMswchb:   ds 1    ; console switch state
ARMinpt4:   ds 1    ; firebutton
ARMvbtime:  ds 1    ; vertical blank time remaining
ARMostime:  ds 1    ; over scan time remaining
; from ARM to 6507
ARMmode:    ds 1    ; 0 = game, 1 = menu, 128 = splash
ARMframe:   ds 1    ; frame counter
ARMaudc0:   ds 1
ARMaudc1:   ds 1
ARMaudf0:   ds 1
ARMaudf1:   ds 1
ARMaudv0:   ds 1
ARMaudv1:   ds 1
; ----- END OF A DATASTREAM -----

MMjoystick:         .byte 0 ; joystick timer
MMfire:             .byte 0 ; fire held
MMselected:         .byte MENU_START_ID
MM_Quadrant:        .byte 0 ;
MM_InitialSector:   .byte 0 ;
MM_Level:           .byte 2 ; 0=kids, 1=beginner, 2=normal, 3=advance
MM_TVtype:          .byte 0 ; 0 = NTSC, 1 = PAL, 2 = SECAM

MainMenuGraphicsRAM: ds MainMenuGraphicsSize
    ; NOTE: only use 3K DD RAM, gave up 1K for use by C Variables
    echo "---- Display Data Menu ", [$B00 - *], "bytes free"

    ; this makes the in-game RAM overlap the Main Menu RAM.
    ORG MainMenuGraphicsRAM
    
    align 4
ClearDatastreamInt:         ; used to rapidly zero the following datastreams    
;-------------- 
        ds 2 * REPOSITION_SIZE  ; part of JumpDataStream
JumpDataStream:
        ds 2 * DATASTREAM_SIZE]
ExitJumpDataStream:        
        ds 2
ScoreDataStream:        
        ds 2* [RADAR_HEIGHT - 6]
ExitScoreDataStream:
        ds 2
;-------------- 
        ds REPOSITION_SIZE  ; part of Player0DataStream
Player0DataStream:
        ds DATASTREAM_SIZE
;-------------- 
        ds REPOSITION_SIZE  ; part of Player1DataStream
Player1DataStream:
        ds DATASTREAM_SIZE
;--------------    
        ds REPOSITION_SIZE  ; part of HMP0DataStream
HMP0DataStream:
        ds DATASTREAM_SIZE + 1
;--------------    
        ds REPOSITION_SIZE  ; part of HMP1DataStream
HMP1DataStream:
        ds DATASTREAM_SIZE + 1
;--------------    
        ds REPOSITION_SIZE  ; part of Missile0DataStream
Missile0DataStream:
        ds DATASTREAM_SIZE + 1
;--------------    
        ds REPOSITION_SIZE  ; part of Missile1DataStream
Missile1DataStream:
        ds DATASTREAM_SIZE + 1
;--------------    
        ds REPOSITION_SIZE  ; part of BallDataStream
BallDataStream:
        ds DATASTREAM_SIZE + 1
;--------------    
        ds REPOSITION_SIZE  ; part of ColorDataStream
ColorDataStream:
        ds DATASTREAM_SIZE
;--------------    
        ds REPOSITION_SIZE  ; part of SizeDataStream
SizeDataStream:
        ds DATASTREAM_SIZE
;--------------    
        
RadarP0DataStream:      ds RADAR_HEIGHT
RadarP1DataStream:      ds RADAR_HEIGHT
FormationP0DataStream:  ds RADAR_HEIGHT
FormationP1DataStream:  ds RADAR_HEIGHT
PF0L:                   ds RADAR_HEIGHT        
PF1L:                   ds RADAR_HEIGHT
PF2L:                   ds RADAR_HEIGHT
PF0R:                   ds RADAR_HEIGHT
PF1R:                   ds RADAR_HEIGHT
PF2R:                   ds RADAR_HEIGHT
;--------------    
    align 4
ClearDatastreamIntSize = * - ClearDatastreamInt   

RadarControl:
        .byte $C8       ; condition color
        .byte HMOVE_0   ; BallXadjust
        ; NOTE: ROM copy of the following data is in stationary_data,
        ;       search for ObjectColorsNTSC
        .byte $18       ; player1, formation color
        .byte SCORE_COLOR ; playfield, score color
        .byte $48       ; player0, enemy color 
        .byte $C8       ; player1, station color
; must directly follow RadarControl so radar colors get updated based on TV-Type
ObjectColors:   
        .byte $28       ; 0  asteroid - orange
        .byte $88       ; 1  mine - blue
        .byte $0c       ; 2  ship - white, subtract 2 due to lumaboost
        .byte $48 ; $38 ; 3  etype - reddish orange
        .byte $58       ; 4  itype - pink
        .byte $ac ; $b8 ; 5  ptype - cyan
        .byte $18       ; 6  spy - yellow
        .byte $c8       ; 7  station -green
        .byte $c4       ; 8  condition green
        .byte $1A       ; 9  condition yellow
        .byte $44       ; 10 condition red    
        .byte $96 ; $88 ; 11 etype lead - blue 
        .byte $38       ; 12 itype lead - orange 
        .byte $58       ; 13 ptype lead - pink
        .byte $48       ; 14 radar enemy / explosion player (first frame, not used)
        .byte $4A       ; 15 explosion player (first frame)
        .byte $1c       ; 16 explosion player (second frame)
        .byte $06       ; 17 explosion player (second frame)
        .byte $1c       ; 18 explosion player (third frame)
        .byte $44       ; 19 explosion player (third frame)
        .byte $1c       ; 20 explosion player (fourth frame)
        .byte $44       ; 21 explosion player (fourth frame)
        .byte $0c       ; 22 explosion mine (first frame)
        .byte $4c       ; 23 explosion mine (first frame)
        .byte $48       ; 24 explosion mine (second frame)
        .byte $58       ; 25 explosion mine (second frame)
        .byte $8A       ; 26 explosion mine (third frame)
        .byte $98       ; 27 explosion mine (third frame)  
        .byte $48       ; 28 explosion station (first frame)
        .byte $4c       ; 29 explosion station (first frame)
        .byte $3a       ; 30 explosion station (second frame)
        .byte $3c       ; 31 explosion station (second frame)
        .byte $1c       ; 32 explosion station (third frame)
        .byte $0c       ; 33 explosion station (third frame) 
        .byte $0C       ; 34 explosion enemy ship (first frame)
        .byte $46       ; 35 explosion enemy ship (first frame)
        .byte $0C       ; 36 explosion enemy ship (second frame)
        .byte $46       ; 37 explosion enemy ship (second frame)
        .byte $0C       ; 38 explosion enemy ship (third frame)
        .byte $46       ; 39 explosion enemy ship (third frame)        
        .byte $08       ; 40 explosion asteroid pod (first frame)
        .byte $2c       ; 41 explosion asteroid pod (first frame)
        .byte $08       ; 42 explosion asteroid pod (second frame)
        .byte $2c       ; 43 explosion asteroid pod (second frame)
        .byte $08       ; 44 explosion asteroid pod (third frame)
        .byte $2c       ; 45 explosion asteroid pod (third frame)
        .byte $46       ; 46 core color red
        .byte $58       ; 47 core color pink
        .byte $4E       ; 48 core color orange
        .byte $5A       ; 49 core color white

StationImages:
        ; extra row is for CollisionQuad()
        ; extra image is for pod removal
        ds (StationHorizontalHeight+1) * 9
                            
;StationCollisionPods:
;        ds StationHorizontalHeight * 8
;StationCollisionStructure:
;        ds StationHorizontalHeight * 8       

    ; NOTE: only use 2.75K DD RAM, gave up 1.25K for use by C Variables
    echo "---- Display Data Game ", [$B00 - *], "bytes free"
    
;===============================================================================
; Cartridge Layout
;-------------------------------------------------------------------------------
; $0000 - $07ff CDF Driver
; $0800 - $0FFF ARM code & data
; $1000 - $1FFF Bank 0, used for ARM code & data
; $2000 - $2FFF Bank 1, used for ARM code & data
; $3000 - $3FFF Bank 2, used for ARM code & data
; $4000 - $4FFF Bank 3, used for ARM code & data
; $5000 - $5FFF Bank 4, used for ARM code & data
; $6000 - $6FFF Bank 5, used for ARM code & data and 6507 menu routines
; $7000 - $7FFF Bank 6, 6507 code, cartridge always boots with this bank active 
;===============================================================================

;===============================================================================
; CDF driver
;-------------------------------------------------------------------------------
; The Harmony/Melody driver is located at Start of Cartridge ROM    
;===============================================================================
    SEG CODE
    ORG $0000

    ; Include CDF Driver
CDF_DRIVER:
;    INCBIN "cdfdriver_v1.bin"
    INCBIN "cdfdriver_v1_1.bin"
CDF_DRIVER_SIZE = * - CDF_DRIVER
    echo "---- CDF DRIVER SIZE", CDF_DRIVER_SIZE, "bytes"
    
;===============================================================================
; ARM code & data
;-------------------------------------------------------------------------------
; 2K of dedicated ARM code space begins at $0800 and expands into the 6507
; banks as needed.
;
;In Draconian it expands into 0-4 as well as the first 11/16 of bank 5.  
;===============================================================================
    ORG $0800
    RORG $0800
    
C_CODE:
    ; Include C code
    INCBIN "./main/bin/testarm.bin"
C_CODE_SIZE = * - C_CODE;
    echo "---- C CODE SIZE", C_CODE_SIZE, "bytes"  
    
;===============================================================================
; Movable ARM data
;-------------------------------------------------------------------------------
;       ARM data that can move around, pointed to by a stationary table
;===============================================================================
;    include "images.asm"
    include "images.dood"
    
PADDING = 15    
    
sample_alert:
    INCBIN "./samples/alert.pds"
sample_alert_size = * - sample_alert 
    ds PADDING, $88  ; sample will play beyond size, so pad with silence
    
sample_battlestations:
    INCBIN "./samples/battlestations.pds"
sample_battlestations_size = * - sample_battlestations 
    ds PADDING, $88  ; sample will play beyond size, so pad with silence
    
sample_blastoff:
    INCBIN "./samples/BLASTOffChipmunk.pds"
sample_blastoff_size = * - sample_blastoff 
    ds PADDING, $88  ; sample will play beyond size, so pad with silence

sample_conditionred:
    INCBIN "./samples/condred3.pds"
sample_conditionred_size = * - sample_conditionred 
    ds PADDING, $88  ; sample will play beyond size, so pad with silence

sample_spyship:
    INCBIN "./samples/SpyShipSighted.pds"
sample_spyship_size = * - sample_spyship 
    ds PADDING, $88  ; sample will play beyond size, so pad with silence
    
    echo "---- samples use ", [* - sample_alert]d
    
STATIONARY_START = $6968    
    echo "----",[STATIONARY_START - *]d , "bytes free for ARM Code, samples, & graphics"

;===============================================================================
; Stationary ARM data
;-------------------------------------------------------------------------------
;       ARM data that shouldn't move when C code changes - tables, etc.
;===============================================================================
    ORG STATIONARY_START
    RORG STATIONARY_START
    

    
   include "stationary_data.asm"   ; 1692 bytes
    
;    include "sfx.asm"
    

    echo "----",[$7000 - *]d , "bytes free for table data left"
    
;===============================================================================
; Bank 6
;-------------------------------------------------------------------------------
;       InitSystem
;       Most 6507 code
;===============================================================================
    ORG $7000
    RORG $F000
    
;===============================================================================
; Call ARM Functions
;-------------------------------------------------------------------------------
; While ARM functions are running a NOP is put on the data bus.  The 6507's PC
; will still be advancing, so call from the start of the 4K Cartridge ROM to
; prevent the PC from wrapping around to zero page as that would crash the
; system. Once the ARM function is finished a JMP ORIGINAL_PC will be put on
; the data bus so 6507 code will resume as expected.
;===============================================================================
CallArmFuncNoAudio:
    ; need to initialize everything before we use digital audio
    ldy #$FF            ; Run ARM code without digital audio
    .byte $0C           ; NOP ABSOLUTE which skips over the LDY #$FE
CallArmFunc:
    ; call with X holding function to run
    ldy #$FE            ; 2 ; Run ARM code with digital audio
    tya                 ; 2  4
    ldy #ARMfunc        ; 2  6
    sty DSPTR           ; 4 10
    ldy #ARMfunc        ; 2 12
    sty DSPTR           ; 4 16
    stx DSWRITE         ; 4 20 
    ldy SWCHA           ; 4 24
    sty DSWRITE         ; 4 28
    ldy SWCHB           ; 4 32
    sty DSWRITE         ; 4 36
    ldy INPT4           ; 4 40
    sty DSWRITE         ; 4 44
    ldy TimeLeftVB      ; 3 47
    sty DSWRITE         ; 4 51
    ldy TimeLeftOS      ; 3 54
    sty DSWRITE         ; 4 58
    tay                 ; 2 60  ; need to restore to Y because
    DIGITAL_AUDIO       ; 5 65  ; this uses A
    sty CALLFN          ; 4 69  ; + unknown # of cycles for exection of ARM code
    rts                 ; 6 75
    
;===============================================================================
; PosObject
;-------------------------------------------------------------------------------
; subroutine for setting the X position of any TIA object
; when called, set the following registers:
;   A - holds the X position of the object
;   X - holds which object to position
;       0 = player0
;       1 = player1
;       2 = missile0
;       3 = missile1
;       4 = ball
; the routine will set the coarse X position of the object, as well as the
; fine-tune register that will be used when HMOVE is used.
;
; Note: The X position differs based on the object, for player0 and player1
;       0 is the leftmost pixel while for missile0, missile1 and ball 1 is
;       the leftmost pixel:
;           players     - X range is 0-159
;           missiles    - X range is 1-160
;           ball        - X range is 1-160
; Note: Setting players to double or quad size will affect the position of
;       the players.
;===============================================================================
PosObject:              ; A holds X value
        pha             ; 3
        sec             ; 2  
        sta WSYNC       ; X holds object, 0=P0, 1=P1, 2=M0, 3=M1, 4=Ball
        DIGITAL_AUDIO   ; 5
        pla             ; 4  9
DivideLoop
        sbc #15         ; 2 11  
        bcs DivideLoop  ; 2 13
        eor #7          ; 2 15
        asl             ; 2 17
        asl             ; 2 19
        sta RESP0,X     ; 4 23 <- set object position
        asl             ; 2 25
        asl             ; 2 27
        sta HMP0,X      ; 4 31
SLEEP12: rts            ; 6 37 
    

;===============================================================================
; Two Color Graphic routine
;-------------------------------------------------------------------------------
; this works by using a "negative" image when drawing the sprites.
; The sprites are colored black while the background is the color
; that shows up as though it were the sprites.  The 2nd color is
; achieved by using the playfield and/or ball.
;
; The missiles (also black) and setting the screen & playfield to BLACK
; is used to hide the screen and playfield beyond the 48 pixel image.
;===============================================================================
ShowTwoColorGraphic:
        sty LoopCounter

S2CGloop:
        sta WSYNC
;---------------------------------------
        DIGITAL_AUDIO       ; 5  5
        Lda #DS_MENU_A      ; 2  7  A
        sta GRP0            ; 3 10
        lda #DS_MENU_A      ; 2 12  B
        sta GRP1            ; 3 15
        lda #DS_MENU_A      ; 2 17  C
        sta GRP0            ; 3 20
        lda #DS_MENU_A      ; 2 22  D
        tax                 ; 2 24
        lda #DS_MENU_A      ; 2 26  E
        tay                 ; 2 28
        lda #DS_MENU_COLOR  ; 2 30
        sta.w COLUPF        ; 4 34
        lda #DS_MENU_COLOR  ; 2 36
        sta COLUBK          ; 3 39
        lda #DS_MENU_A      ; 2 41  F
        stx GRP1            ; 3 44
        sty GRP0            ; 3 47
        sta GRP1            ; 3 50
        sta GRP0            ; 3 53
        ldx #0              ; 2 55
        stx COLUBK          ; 3 58
        stx COLUPF          ; 3 61
        dec LoopCounter     ; 5 66
        bne S2CGloop        ; 3 69      <- OK if page crossed
        stx PF0             ; 3 72
        stx PF1             ; 3 75
;---------------------------------------
        stx GRP0            ; 3 78/2
        DIGITAL_AUDIO       ; 5  7        
        stx GRP1            ; 3 10
        stx GRP0            ; 3 13
        rts                 ; 6 19       
        
        
;===============================================================================
; Start
;-------------------------------------------------------------------------------
; This routine is run when the Atari is first powered on
;===============================================================================
Start:
; normally we'd use CLEAN_START, but we want to detect console
; detection logic is to check the values in ZP RAM locations $D0 and $D1
;
;   if $D0 contains $2C and $D1 contains $A9 then
;       system = 7800           // game was loaded from Harmony menu on a 7800
;   else if both contain $00 then
;       system = ZP RAM $80     // game was flashed to Harmony/Melody so CDF
;                               // driver checked $D0 and $D1 for us
;   else
;       system = 2600           // game was loaded from Harmony menu on a 2600

; start of console detection routine
        sei 
        cld
        
        ldy #0              ; assume system = 2600        
        ldx $d0
        beq .confirmFlashed ; if $00 then game might be flashed on Harmony/Melody
        cpx #$2c
        bne .is2600         ; if not $2C then loaded via Harmony Menu on 2600        
        ldx $d1
        cpx #$a9            ; <-- evil7800 note - this A9 is OK as FastFetch is not active
        bne .is2600
        dey                 ; 7800: y=$FF
        bne .done           ; this will always branch
        
.confirmFlashed        
        ldx $d1
        bne .is2600         ; if not $00 then loaded via Harmony Menu on 2600
        ldy $80             ;
        
.is2600                     ; 2600: y == 0
.done                       ; 7800: y != 0
; end of console detection routine

        ldx #0
        txa
CLEAR_STACK:
        dex
        txs
        pha
        bne CLEAR_STACK     ; SP=$FF, X = A = 0
        
        sty TimeLeftVB ; during Init this is used to pass 2600/7800 detection to ARM
        
        ; TURN ON SAMPLES and FAST FETCH/JUMP
        ldx #AUDIOSAMPLE + FASTON
        stx SETMODE

        ldx #RUN_INIT
        jsr CallArmFuncNoAudio
        lda #DSCOMM       ; datastream 32 can be read
        sta Mode
        jmp VerticalBlank
        
OverScan:
        jsr CallArmFunc
        lda #DSCOMM
        sta Mode
        lda #DSCOMM
        sta Frame
        
        lda INTIM
        sta TimeLeftOS        
    
OSwait:
        sta WSYNC
        DIGITAL_AUDIO       ; 5        
        lda INTIM
        bpl OSwait
        
VerticalBlank:
        ldx #2
        stx WSYNC    
        stx VSYNC           ; 3  3
        stx VBLANK          ; 3  6
        DIGITAL_AUDIO       ; 5 11       
        ldy #53-1 ; 47        ; 2 13 give extra processing time to VB
        sty TIM64T          ; 4 17
        sty WSYNC           ; line 1 of VSYNC
        DIGITAL_AUDIO       ; 5        
        lda #DSCOMM
        sta AUDC0
        lda #DSCOMM
        sta AUDC1
        lda #DSCOMM
        sta AUDF0
        lda #DSCOMM
        sta AUDF1
        lda #DSCOMM
;        sta AUDV0      ; this conflicts with DIGITAL_AUDIO's updates of AUDV0
        lda #DSCOMM
        sta AUDV1
        sty WSYNC       ; line 2 of VSYNC     
        DIGITAL_AUDIO       ; 5        
        ldx #0          ; 2  2
        stx VDELP0      ; 3  5
        stx VDELP1      ; 3  8
        stx VDELBL      ; 3 11
        stx ENABL       ; 3 14
        stx ENAM1       ; 3 17
        stx ENAM0       ; 3 20
        stx GRP0        ; 3 23
        stx GRP1        ; 3 26
        stx PF0         ; 3 29
        stx PF1         ; 3 32
        stx PF2         ; 3 35
        stx COLUBK      ; 3 38
        stx COLUPF      ; 3 41
        stx CTRLPF      ; 3 44
        stx NUSIZ0      ; 3 47
        stx NUSIZ1      ; 3 50
        stx WSYNC       ; line 3 of VSYNC
        stx VSYNC       ; turn off VSYNC
        DIGITAL_AUDIO       ; 5
        
        lda Mode        ; 0 = game, 1 = menu, $80 = splash
        beq VBG         
        bpl VBM
        ldx #RUN_SPLASH_VB
        .byte $2c
VBM:    ldx #RUN_MENU_VB
        .byte $2c
VBG:    ldx #RUN_GAME_VB
        jsr CallArmFunc
        
        lda Mode
        beq GameVB
        bpl .skip
        jmp SplashVB   
.skip:        
        jmp MainMenuVB
        
GameVB:       
        ; update audio now as we don't know when CalArmFunc last updated it
        DIGITAL_AUDIO       ; 5  5
        ldx #3              ; 2  7
        stx VDELP0          ; 3 10
        stx VDELBL          ; 3 13
        ldx #15             ; 2 15
        stx COLUP0          ; 3 18
        stx COLUP1          ; 3 21
        
        ldy #0
        lda INTIM
        sta TimeLeftVB   
        clv                 ; clear overflow flag, kernel uses bvc to save ROM
        
GameVBwait:
        sta WSYNC           ;
        DIGITAL_AUDIO       ; 5        
        lda INTIM           ; 4  9
        bpl GameVBwait      ; 2 11
        sty VBLANK          ; 3 14 Video Blank OFF (thus image output on)
        sta HMCLR           ; 3 17
        jmp FASTJMP         ; 3 20
        
        ; For a given stream position:
        ; SIZE, COLOR, and HMxx values are for the CURRENT scanline
        ; GRP0, GRP1, and ENAxx values are for the NEXT scanline

NormalKernel:               ;   20
        lda #DS_SIZE        ; 2 22 <- just to keep stream in sync
nk1:    lda #DS_COLOR       ; 2 24 2 33 from Resm0Strobe28 <- just to keep stream in sync
        lda #DS_HMP0        ; 2 26
nk2:    sta HMP0            ; 3 29 3 39 from Resm0Strobe33
        lda #DS_HMP1        ; 2 31
nk3:    sta HMP1            ; 3 34 3 44 from Resm0Strobe38       
        lda #DS_MISSILE0    ; 2 36
nk4:    tax                 ; 2 38 2 48 from Resm0Strobe43 
        stx HMM0            ; 3 41
nk5:    lda #DS_MISSILE1    ; 2 43 2 53 from Resm0Strobe48
        tay                 ; 2 45
        sty HMM1            ; 3 48
        lda #DS_BALL        ; 2 50
nk6:    sta HMBL            ; 3 53 3 62 from Resm0Strobe53             
        sta ENABL           ; 3 56 <- for next scanline, VDELBL on 
nk8:    lda #DS_GRP0        ; 2 58 2 68 from Resm0Strobe63
nk7:    sta GRP0            ; 3 61 3 71 from Resm0Strobe58 <- for next scanline, VDELP0 on
nk9:    lda #DS_GRP1        ; 2 63 2 73 from Resm0Strobe68
        sta WSYNC           ; 3 66/0
nk10:   sta HMOVE           ; 3  3
        sta GRP1            ; 3  6 <- also updates GRP0 and BL
        DIGITAL_AUDIO       ; 5 11
        stx ENAM0           ; 3 14
        sty ENAM1           ; 3 17
        jmp FASTJMP         ; 3 20
        
InitLevelKernel:                ; screen jitter occurs if InitLevel() is run during
        ldx #189                ; OverScan, so this is set up to run it mid-screen
        stx TIM64T
;        ldx #WHITE              ; for testing, make background white to show how
;        stx COLUBK              ; long the InitLevel takes to run
        ldx #RUN_INIT_LEVEL
        jsr CallArmFunc
;        ldx #0
;        stx COLUBK
ILwait:
        sta WSYNC               ; 
        bit TIMINT              ; 4  4
        bpl ILwait              ; 2  6
        jsr SLEEP12             ;12 18
        SLEEP 2                 ; 2 20

ExitKernel:                     ;   20
        ; currently drawing the last line of the gameplay area
        ldx #0                  ; 2 22
        stx GRP0                ; 3 25 <- for next scanline, VDELP0 on
        stx ENABL               ; 3 28 <- for next scanline, VDELBL on               
        sta WSYNC               ; 3 31/0
        
        ; blank line before condition color
        stx GRP1                ; 3  3 <- also updates GRP0 and BL
        stx ENAM0               ; 3  6
        DIGITAL_AUDIO           ; 5 11
        stx HMCLR               ; 3 14
        stx ENAM1               ; 3 17
        stx NUSIZ0              ; 3 20
        stx NUSIZ1              ; 3 23
        ldx #HMOVE_R2           ; 2 25
        stx HMBL                ; 3 28
        lda #DS_RADAR_CONTROL   ; 2 30 <- status color
        tax                     ; 2 32 <- in X for duration of status display
        ldy #4                  ; 2 34
        sty CTRLPF              ; 3 37 <- 1x ball, display priority over player
        ldy #HMOVE_R8           ; 2 39
        sty HMP0                ; 3 42 <- finetune for radar
        ldy #HMOVE_L6           ; 2 44
        sty HMP1                ; 3 47 <- finetune for radar
        dec Sleep5              ; 5 52
        SLEEP 2                 ; 2 54
        sty RESP0               ; 3 57
        sty RESBL               ; 3 60
        sty RESP1               ; 3 63
        sta WSYNC               ; 3 66/0
        
        ; condition line color
        sta HMOVE               ; 3  3
        stx COLUBK              ; 3  6
        DIGITAL_AUDIO           ; 5 11
        stx COLUP0              ; 3 14
        stx COLUP1              ; 3 17
        sta WSYNC
        
        ; first line of top radar frame
        ldy #0                  ; 2  2
        sty COLUBK              ; 3  5
        DIGITAL_AUDIO           ; 5 10
        lda #DS_RADAR_GRP0      ; 2 12
        sta GRP0                ; 3 15
        lda #DS_RADAR_GRP1      ; 2 17
        sta GRP1                ; 3 20
        ldy #RADAR_HEIGHT - 5   ; 2 22
        sty LoopCounter         ; 3 25
        lda #DS_RADAR_CONTROL   ; 2 27 <- position player in radar
        sta HMBL                ; 3 30
        lda #DS_RADAR_CONTROL   ; 2 32
        sta FormationColor      ; 3 35
        sta WSYNC
        
        ; second line of top radar frame
        lda #<DS_RADAR_GRP0     ; 2  2
        sta GRP0                ; 3  5
        DIGITAL_AUDIO           ; 5 10
        lda #<DS_RADAR_GRP1     ; 2 12
        sta GRP1                ; 3 15
        ldy #HMOVE_R2           ; 2 17
        sty HMP0                ; 3 20
        ldy #HMOVE_L2           ; 2 22
        sty HMP1                ; 3 25
        lda #DS_RADAR_CONTROL   ; 2 27
        sta COLUPF              ; 3 30     
        jsr SLEEP12             ;12 42
        jsr SLEEP12             ;12 54
        jsr SLEEP12             ;12 66
        dec Sleep5              ; 5 71
        SLEEP 2                 ; 2 73        
        ; first line of score/radar
        sta HMOVE               ; 3 76/0
        DIGITAL_AUDIO           ; 5  5
        lda #<DS_PF0L           ; 2  7
        sta PF0                 ; 3 10
        sta ENABL               ; 3 13 - on VDEL
        lda #<DS_PF1L           ; 2 15
        sta PF1                 ; 3 18
        lda #<DS_PF2L           ; 2 20
        sta PF2                 ; 3 23
        lda #<DS_RADAR_GRP0     ; 2 25
        sta GRP0                ; 3 28 - on VDEL
        lda #<DS_RADAR_GRP1     ; 2 30
        sta GRP1                ; 3 33 - updates GRP0 and BL as well
        lda #<DS_RADAR_CONTROL  ; 2 35
        sta COLUP0              ; 3 38 - enemy color in radar        
; TODO        
; change this to lda #<DS_STATION_COLOR and eliminate entry in radar control
; TODO
; change the formation color and enemy color to work the same
        lda #<DS_RADAR_CONTROL  ; 2 41
        sta COLUP1              ; 3 43 - station color in radar
        lda #<DS_PF0R           ; 2 45
        sta PF0                 ; 3 48 - PF0R, 28-49
        lda #<DS_PF1R           ; 2 50
        sta PF1                 ; 3 53
        lda #<DS_PF2R           ; 2 55
        sta PF2                 ; 3 58 
        ldy #HMOVE_L2           ; 2 60 - prep to shift players to draw the
        sty HMP0                ; 3 63 - frame AFTER ScoreLoop is done
        ldy #HMOVE_R2           ; 2 65
        sty HMP1                ; 3 68
        ldy #2                  ; 2 70
        sty NUSIZ0              ; 3 73 - 2 copies medium for formation display 
        sty NUSIZ1              ; 3 76 - 2 copies medium for formation display
        
ScoreKernel:
        lda #<DS_PF0L           ; 2  2
        sta PF0                 ; 3  5
        sta ENABL               ; 3  8 - on VDEL
        DIGITAL_AUDIO           ; 5 13
        lda #<DS_PF1L           ; 2 15
        sta PF1                 ; 3 18
        lda #<DS_PF2L           ; 2 20
        sta PF2                 ; 3 23
        lda #<DS_RADAR_GRP0     ; 2 25
        sta GRP0                ; 3 28 - on VDEL
        lda #<DS_RADAR_GRP1     ; 2 30
        sta GRP1                ; 3 33 - updates GRP0 and BL as well
        lda #<DS_STATION_COLOR  ; 2 35
        sta COLUP1              ; 3 38
        lda #<DS_FORMATION_GRP0 ; 2 40
        sta GRP0                ; 3 43 - on VDEL
        lda #<DS_PF0R           ; 2 45
        sta PF0                 ; 3 48 - PF0R, 28-49
        lda #<DS_PF1R           ; 2 50
        sta PF1                 ; 3 53 - PF1R, 39-54
        lda #DS_FORMATION_GRP1  ; 2 55
        tay                     ; 2 57
        lda #<DS_PF2R           ; 2 59
        sta PF2                 ; 3 62 - PF2R, 50-65
        lda FormationColor      ; 3 65
        sty GRP1                ; 3 68 - updates GRP0 as well
        sta COLUP1              ; 3 71
        SLEEP 2                 ; 2 73
        jmp FASTJMP             ; 3 76
        
ExitScoreKernel:        
        sta HMOVE
        DIGITAL_AUDIO
        ldy #0
        sty PF0
        sty PF1
        sty PF2
        sty ENABL
        sty NUSIZ0              ; 1 copy, no more formation display
        sty NUSIZ1              ; 1 copy, no more formation display
        lda #<DS_RADAR_GRP0
        sta GRP0
        lda #<DS_RADAR_GRP1
        sta GRP1
        stx COLUP0
        stx COLUP1      
        
        ; second line of bottom radar frame
        sta WSYNC
        DIGITAL_AUDIO
        lda #<DS_RADAR_GRP0
        sta GRP0
        lda #<DS_RADAR_GRP1
        sta GRP1       

GameKernelDone:   
        stx WSYNC
        DIGITAL_AUDIO       ; 5
        ldx #2
        stx VBLANK  ; Video Blank ON (thus image output off)
        ldx #33+1
        stx TIM64T
        ldx #RUN_GAME_OS
        jmp OverScan
        
        ; NOTE: align 2 inside missile_graphics.asm
        include "missile_graphics.asm"  ; 222 bytes

        
 echo "----",[((* & $ff00) + $100) - *]d, "bytes before FirstResp0"  
        align 256
        
; due to how the kernels are used in the C code, the reposition kernels must
; be in the following order:
;       Resbl...
;       Resp0...
;       Resp1...
;       Resm0...
;       Resm1...
; the kernels within each group can be in any order, the FirstRes## label
; must be with the first of each object.

FirstResbl:        
ResblStrobe23:                  ;   20
        sta RESBL               ; 3 23
;        SLEEP 3                 ; 3 26
;        lda #DS_SIZE            ; 2 28
;        sta CTRLPF              ; 3 31  <- changes player size
        tsx                     ; 2 25  SP=$FF OMEGA save 1 byte
        lda #DS_SIZE            ; 2 27
        sta CTRLPF-$FF,x        ; 4 31
FBL28:  lda #DS_COLOR           ; 2 33
        sta COLUPF              ; 3 36
FBL33:  lda #DS_HMP0            ; 2 38
        sta HMP0                ; 3 41
FBL38:  lda #DS_HMP1            ; 2 43
        sta HMP1                ; 3 46        
FBL43:  lda #DS_GRP0            ; 2 48
        sta GRP0                ; 3 51  <- for next scanline, VDELP0 on
FBL48:  lda #DS_BALL            ; 2 53
        sta ENABL               ; 3 56  <- for next scanline, VDELBL on
FBL53:  sta HMBL                ; 3 59
        lda #DS_MISSILE0        ; 2 61
FBL58:  tax                     ; 2 63
        stx HMM0                ; 3 66
FBL63:  lda #DS_MISSILE1        ; 2 68
        tay                     ; 2 70
        sty.w HMM1              ; 4 74
        lda #DS_GRP1            ; 2 76
FBL68_73:        
        sta HMOVE               ; 3  3
        sta GRP1                ; 3  6  @0-21
        DIGITAL_AUDIO           ; 5 11
        stx ENAM0               ; 3 14  @0-21
        sty ENAM1               ; 3 17  @0-21
        jmp FASTJMP             ; 3 20         
        
ResblStrobe28:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta CTRLPF              ; 3 25  <- changes player size
        sta RESBL               ; 3 28
        sbvc FBL28              ; 3 31
;        jmp FBL28               ; 3 31

ResblStrobe33:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta CTRLPF              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUPF              ; 3 30
        sta RESBL               ; 3 33
        sbvc FBL33              ; 3 36
;        jmp FBL33               ; 3 36
        
ResblStrobe38:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta CTRLPF              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUPF              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        sta RESBL               ; 3 38
        sbvc FBL38              ; 3 41
;        jmp FBL38               ; 3 41
        
ResblStrobe43:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta CTRLPF              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUPF              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        sta RESBL               ; 3 43
        sbvc FBL43              ; 3 46
;        jmp FBL43               ; 3 46
        
ResblStrobe48:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta CTRLPF              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUPF              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        lda #DS_GRP0            ; 2 42
        sta GRP0                ; 3 45  <- for next scanline, VDELP0 on
        sta RESBL               ; 3 48
        sbvc FBL48              ; 3 51
;        jmp FBL48               ; 3 51
        
ResblStrobe53:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta CTRLPF              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUPF              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        lda #DS_GRP0            ; 2 42
        sta GRP0                ; 3 45  <- for next scanline, VDELP0 on
        lda #DS_BALL            ; 2 47
        sta ENABL               ; 3 50
        sta RESBL               ; 3 53
;        sbvc FBL53              ; 3 56 too far
        jmp FBL53               ; 3 56        
        
ResblStrobe58:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta CTRLPF              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUPF              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        lda #DS_GRP0            ; 2 42
        sta GRP0                ; 3 45  <- for next scanline, VDELP0 on
        lda #DS_BALL            ; 2 47
        sta ENABL               ; 3 50
        sta HMBL                ; 3 53
        lda #DS_MISSILE0        ; 2 55
        sta RESBL               ; 3 58
;        sbvc FBL58              ; 3 61 too far
        jmp FBL58               ; 3 61   
        
ResblStrobe63:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta CTRLPF              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUPF              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        lda #DS_GRP0            ; 2 42
        sta GRP0                ; 3 45  <- for next scanline, VDELP0 on
        lda #DS_BALL            ; 2 47
        sta ENABL               ; 3 50
        sta HMBL                ; 3 53
        lda #DS_MISSILE0        ; 2 55
        tax                     ; 2 57
        stx HMM0                ; 3 60
        sta RESBL               ; 3 63
;        sbvc FBL63              ; 3 66 too far
        jmp FBL63               ; 3 66   
        
ResblStrobe68:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta CTRLPF              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUPF              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        lda #DS_GRP0            ; 2 42
        sta GRP0                ; 3 45  <- for next scanline, VDELP0 on
        lda #DS_BALL            ; 2 47
        sta ENABL               ; 3 50
        sta HMBL                ; 3 53
        lda #DS_MISSILE0        ; 2 55
        tax                     ; 2 57
        stx HMM0                ; 3 60
        lda #DS_MISSILE1        ; 2 62
        tay                     ; 2 64
        sta.w RESBL             ; 4 68
        sty HMM1                ; 3 71
        lda #DS_GRP1            ; 2 73
        jmp FBL68_73
        
ResblStrobe73:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta CTRLPF              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUPF              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        lda #DS_GRP0            ; 2 42
        sta GRP0                ; 3 45  <- for next scanline, VDELP0 on
        lda #DS_BALL            ; 2 47
        sta ENABL               ; 3 50
        sta HMBL                ; 3 53
        lda #DS_MISSILE0        ; 2 55
        tax                     ; 2 57
        stx HMM0                ; 3 60
        lda #DS_MISSILE1        ; 2 62
        tay                     ; 2 64   
        sty HMM1                ; 3 67
        lda #DS_GRP1            ; 2 69
        sta.w RESBL             ; 4 73
        jmp FBL68_73               ; 3 76
       
FirstResp0:        

Resp0Strobe23:                  ;   20
        sta RESP0               ; 3 23
;         SLEEP 3                 ; 3 26
;         lda #DS_SIZE            ; 2 28
;         sta NUSIZ0              ; 3 31  <- changes player size

        tsx                     ; 2 25  SP=$FF    OMEGA RESP0 (1) - save 1 byte
        lda #DS_SIZE            ; 2 27
        sta NUSIZ0-$FF,X        ; 4 31  <- changes player size

FP028:  lda #DS_COLOR           ; 2 33
        sta COLUP0              ; 3 36
FP033:  lda #DS_HMP0            ; 2 38
        sta HMP0                ; 3 41
FP038:  lda #DS_HMP1            ; 2 43
        sta HMP1                ; 3 46
FP043:  lda #DS_GRP0            ; 2 48
        sta GRP0                ; 3 51  <- for next scanline, VDELP0 on
FP048:  lda #DS_BALL            ; 2 53
        sta ENABL               ; 3 56  <- for next scanline, VDELBL on
FP053:  sta HMBL                ; 3 59
        lda #DS_MISSILE0        ; 2 61
FP058:  tax                     ; 2 63
        stx HMM0                ; 3 66
FP063:  lda #DS_MISSILE1        ; 2 68
        tay                     ; 2 70
;         sty HMM1                ; 3 73
;         lda #DS_GRP1            ; 2 75
;         sta.w HMOVE             ; 4 79/3
;         sta GRP1                ; 3  6  @0-21
;         DIGITAL_AUDIO           ; 5 11
;         stx ENAM0               ; 3 14  @0-21
;         sty ENAM1               ; 3 17  @0-21
;         jmp FASTJMP             ; 3 20
        sty.w HMM1              ; 4 74     OMEGA RESP0 (10 - 11)
        lda #DS_GRP1            ; 2 76
FP068_73:
        sta HMOVE               ; 3 79/3
        sta GRP1                ; 3  6  @0-21
        DIGITAL_AUDIO           ; 5 11
        stx ENAM0               ; 3 14  @0-21
        sty ENAM1               ; 3 17  @0-21
        jmp FASTJMP             ; 3 20

Resp0Strobe28:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes player size
        sta RESP0               ; 3 28
;         jmp FP028               ; 3 31
        sbvc FP028              ; 3 31  always branch   OMEGA RESP0 (2) - save 1 byte

Resp0Strobe33:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUP0              ; 3 30
        sta RESP0               ; 3 33
;         jmp FP033               ; 3 36
        sbvc FP033              ; 3 36  always branch   OMEGA RESP0 (3) - save 1 byte

Resp0Strobe38:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUP0              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        sta RESP0               ; 3 38
;         jmp FP038               ; 3 41
        sbvc FP038              ; 3 41  always branch   OMEGA RESP0 (4) - save 1 byte

Resp0Strobe43:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUP0              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        sta RESP0               ; 3 43
;         jmp FP043               ; 3 46
        sbvc FP043              ; 3 46  always branch   OMEGA RESP0 (5) - save 1 byte

Resp0Strobe48:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUP0              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        lda #DS_GRP0            ; 2 42
        sta GRP0                ; 3 45  <- for next scanline, VDELP0 on
        sta RESP0               ; 3 48
;         jmp FP048               ; 3 51
        sbvc FP048              ; 3 51  always branch   OMEGA RESP0 (6) - save 1 byte

Resp0Strobe53:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUP0              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        lda #DS_GRP0            ; 2 42
        sta GRP0                ; 3 45  <- for next scanline, VDELP0 on
        lda #DS_BALL            ; 2 47
        sta ENABL               ; 3 50
        sta RESP0               ; 3 53
        jmp FP053               ; 3 56
;        sbvc FP053              ; 3 56  too far always branch   OMEGA RESP0 (7) - save 1 byte


Resp0Strobe58:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUP0              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        lda #DS_GRP0            ; 2 42
        sta GRP0                ; 3 45  <- for next scanline, VDELP0 on
        lda #DS_BALL            ; 2 47
        sta ENABL               ; 3 50
        sta HMBL                ; 3 53
        lda #DS_MISSILE0        ; 2 55
        sta RESP0               ; 3 58
        jmp FP058               ; 3 61
;         sbvc FP058              ; 3 61  too faralways branch   OMEGA RESP0 (8) - save 1 byte

Resp0Strobe63:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUP0              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        lda #DS_GRP0            ; 2 42
        sta GRP0                ; 3 45  <- for next scanline, VDELP0 on
        lda #DS_BALL            ; 2 47
        sta ENABL               ; 3 50
        sta HMBL                ; 3 53
        lda #DS_MISSILE0        ; 2 55
        tax                     ; 2 57
        stx HMM0                ; 3 60
        sta RESP0               ; 3 63
        jmp FP063               ; 3 66
;        sbvc FP063              ; 3 66  too far always branch   OMEGA RESP0 (9) - save 1 byte

Resp0Strobe68:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUP0              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        lda #DS_GRP0            ; 2 42
        sta GRP0                ; 3 45  <- for next scanline, VDELP0 on
        lda #DS_BALL            ; 2 47
        sta ENABL               ; 3 50
        sta HMBL                ; 3 53
        lda #DS_MISSILE0        ; 2 55
        tax                     ; 2 57
        stx HMM0                ; 3 60
        lda #DS_MISSILE1        ; 2 62
        tay                     ; 2 64
        sta.w RESP0             ; 4 68
;         SLEEP 2                 ; 2 70
;         sty HMM1                ; 3 73
;         lda #DS_GRP1            ; 2 75
;         sta.w HMOVE             ; 4 79/3
;         sta GRP1                ; 3  6  @0-21
;         DIGITAL_AUDIO           ; 5 11
;         stx ENAM0               ; 3 14  @0-21
;         sty ENAM1               ; 3 17  @0-21
;         jmp FASTJMP             ; 3 20

        sty HMM1               ; 3 71
        lda #DS_GRP1           ; 2 73
        jmp FP068_73; 3 76   always branch     OMEGA RESP0 (10) - save 14 bytes

Resp0Strobe73:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUP0              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        lda #DS_GRP0            ; 2 42
        sta GRP0                ; 3 45  <- for next scanline, VDELP0 on
        lda #DS_BALL            ; 2 47
        sta ENABL               ; 3 50
        sta HMBL                ; 3 53
        lda #DS_MISSILE0        ; 2 55
        tax                     ; 2 57
        stx HMM0                ; 3 60
        lda #DS_MISSILE1        ; 2 62
        tay                     ; 2 64
        sty HMM1                ; 3 67
        lda #DS_GRP1            ; 2 69
        sta.w RESP0             ; 4 73
;         SLEEP 2                 ; 2 75
;         sta.w HMOVE             ; 4 79/3
;         sta GRP1                ; 3  6  @0-21
;         DIGITAL_AUDIO           ; 5 11
;         stx ENAM0               ; 3 14  @0-21
;         sty ENAM1               ; 3 17  @0-21
;         jmp FASTJMP             ; 3 20

        jmp FP068_73; 3 76  always branch   OMEGA RESP0 (11) - save 15 bytes
        
 
     
FirstResp1:      
Resp1Strobe63:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUP1              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        lda #DS_GRP0            ; 2 42
        sta GRP0                ; 3 45  <- for next scanline, VDELP0 on
        lda #DS_BALL            ; 2 47
        sta ENABL               ; 3 50
        sta HMBL                ; 3 53
        lda #DS_MISSILE0        ; 2 55
        tax                     ; 2 57
        stx HMM0                ; 3 60
        sta RESP1               ; 3 63
        jmp FP163               ; 3 66   
        ;sbvc FP163             different page
        
Resp1Strobe68:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUP1              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        lda #DS_GRP0            ; 2 42
        sta GRP0                ; 3 45  <- for next scanline, VDELP0 on
        lda #DS_BALL            ; 2 47
        sta ENABL               ; 3 50
        sta HMBL                ; 3 53
        lda #DS_MISSILE0        ; 2 55
        tax                     ; 2 57
        stx HMM0                ; 3 60
        lda #DS_MISSILE1        ; 2 62
        tay                     ; 2 64
        sta.w RESP1             ; 4 68
        sty HMM1                ; 3 71
        lda #DS_GRP1            ; 2 73
        jmp FP168_73            ; 3 76   
        ;sbvc FP168_73          different page
        ; SLEEP 2                 ; 2 70
        ; sty HMM1                ; 3 73
        ; lda #DS_GRP1            ; 2 75
        ; sta.w HMOVE             ; 4 79/3
        ; sta GRP1                ; 3  6  @0-21
        ; DIGITAL_AUDIO           ; 5 11
        ; stx ENAM0               ; 3 14  @0-21
        ; sty ENAM1               ; 3 17  @0-21
        ; jmp FASTJMP             ; 3 20       
        
Resp1Strobe73:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUP1              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        lda #DS_GRP0            ; 2 42
        sta GRP0                ; 3 45  <- for next scanline, VDELP0 on
        lda #DS_BALL            ; 2 47
        sta ENABL               ; 3 50
        sta HMBL                ; 3 53
        lda #DS_MISSILE0        ; 2 55
        tax                     ; 2 57
        stx HMM0                ; 3 60
        lda #DS_MISSILE1        ; 2 62
        tay                     ; 2 64   
        sty HMM1                ; 3 67
        lda #DS_GRP1            ; 2 69
        sta.w RESP1             ; 4 73
        ;jmp FP168_73            ; 3 76
        sbvc FP168_73
        ; SLEEP 2                 ; 2 75
        ; sta.w HMOVE             ; 4 79/3
        ; sta GRP1                ; 3  6  @0-21
        ; DIGITAL_AUDIO           ; 5 11
        ; stx ENAM0               ; 3 14  @0-21
        ; sty ENAM1               ; 3 17  @0-21
        ; jmp FASTJMP             ; 3 20  
        
Resp1Strobe53:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUP1              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        lda #DS_GRP0            ; 2 42
        sta GRP0                ; 3 45  <- for next scanline, VDELP0 on
        lda #DS_BALL            ; 2 47
        sta ENABL               ; 3 50
        sta RESP1               ; 3 53
        ;jmp FP153               ; 3 56
        sbvc FP153              ; 3 56 too far
        
Resp1Strobe58:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUP1              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        lda #DS_GRP0            ; 2 42
        sta GRP0                ; 3 45  <- for next scanline, VDELP0 on
        lda #DS_BALL            ; 2 47
        sta ENABL               ; 3 50
        sta HMBL                ; 3 53
        lda #DS_MISSILE0        ; 2 55
        sta RESP1               ; 3 58
        ;jmp FP158               ; 3 61
        sbvc FP158              ; 3 61        
        
Resp1Strobe23:                  ;   20
        sta RESP1               ; 3 23
;        SLEEP 3                 ; 3 26
;        lda #DS_SIZE            ; 2 28
;        sta NUSIZ1              ; 3 31  <- changes player size
        tsx                     ; 2 25  SP=$FF    OMEGA RESP0 (1) - save 1 byte
        lda #DS_SIZE            ; 2 27
        sta NUSIZ1-$FF,X        ; 4 31  <- changes player size
        
FP128:  lda #DS_COLOR           ; 2 33
        sta COLUP1              ; 3 36
FP133:  lda #DS_HMP0            ; 2 38
        sta HMP0                ; 3 41
FP138:  lda #DS_HMP1            ; 2 43
        sta HMP1                ; 3 46        
FP143:  lda #DS_GRP0            ; 2 48
        sta GRP0                ; 3 51  <- for next scanline, VDELP0 on
FP148:  lda #DS_BALL            ; 2 53
        sta ENABL               ; 3 56  <- for next scanline, VDELBL on
FP153:  sta HMBL                ; 3 59
        lda #DS_MISSILE0        ; 2 61
FP158:  tax                     ; 2 63
        stx HMM0                ; 3 66
FP163:  lda #DS_MISSILE1        ; 2 68
        tay                     ; 2 70
        sty.w HMM1              ; 4 74
        lda #DS_GRP1            ; 2 76
FP168_73:        
        sta HMOVE               ; 3  3
        sta GRP1                ; 3  6  @0-21
        DIGITAL_AUDIO           ; 5 11
        stx ENAM0               ; 3 14  @0-21
        sty ENAM1               ; 3 17  @0-21
        jmp FASTJMP             ; 3 20         
        
Resp1Strobe28:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes player size
        sta RESP1               ; 3 28
        ;jmp FP128               ; 3 31
        sbvc FP128              ; 3 31

Resp1Strobe33:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUP1              ; 3 30
        sta RESP1               ; 3 33
        ;jmp FP133               ; 3 36
        sbvc FP133              ; 3 36
        
Resp1Strobe38:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUP1              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        sta RESP1               ; 3 38
        ;jmp FP138               ; 3 41
        sbvc FP138              ; 3 41
        
Resp1Strobe43:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUP1              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        sta RESP1               ; 3 43
        ;jmp FP143               ; 3 46
        sbvc FP143              ; 3 46
        
Resp1Strobe48:                  ;   20   
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes player size
        lda #DS_COLOR           ; 2 27
        sta COLUP1              ; 3 30
        lda #DS_HMP0            ; 2 32
        sta HMP0                ; 3 35
        lda #DS_HMP1            ; 2 37
        sta HMP1                ; 3 40
        lda #DS_GRP0            ; 2 42
        sta GRP0                ; 3 45  <- for next scanline, VDELP0 on
        sta RESP1               ; 3 48
        ;jmp FP148               ; 3 51
        sbvc FP148              ; 3 51
        

        
        
FirstResm0        
Resm0Strobe23:                  ;   20
        sta RESM0               ; 3 23
        lda #DS_SIZE            ; 2 25
        sta NUSIZ0              ; 3 28  <- changes missile size
        jmp nk1                 ; 3 31
        
Resm0Strobe28:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes missile size
        sta RESM0               ; 3 28
        jmp nk1                 ; 3 31
     
Resm0Strobe33:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes missile size
        lda #DS_COLOR           ; 2 27
        lda #DS_HMP0            ; 2 29
        sta.w RESM0             ; 4 33
        jmp nk2                 ; 3 36
        
Resm0Strobe38:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes missile size
        lda #DS_COLOR           ; 2 27
        lda #DS_HMP0            ; 2 29
        sta HMP0                ; 3 32
        lda #DS_HMP1            ; 2 34
        sta.w RESM0             ; 4 38
        jmp nk3                 ; 3 41
        
Resm0Strobe43:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes missile size
        lda #DS_COLOR           ; 2 27
        lda #DS_HMP0            ; 2 29
        sta HMP0                ; 3 32
        lda #DS_HMP1            ; 2 34
        sta HMP1                ; 3 37
        lda #DS_MISSILE0        ; 2 39
        sta.w RESM0             ; 4 43
        jmp nk4                 ; 3 46
        
Resm0Strobe48:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes missile size
        lda #DS_COLOR           ; 2 27
        lda #DS_HMP0            ; 2 29
        sta HMP0                ; 3 32
        lda #DS_HMP1            ; 2 34
        sta HMP1                ; 3 37
        lda #DS_MISSILE0        ; 2 39
        tax                     ; 2 41
        stx HMM0                ; 3 44
        sta.w RESM0             ; 4 48
        jmp nk5                 ; 3 51        
        
Resm0Strobe53:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes missile size
        lda #DS_COLOR           ; 2 27
        lda #DS_HMP0            ; 2 29
        sta HMP0                ; 3 32
        lda #DS_HMP1            ; 2 34
        sta HMP1                ; 3 37
        lda #DS_MISSILE0        ; 2 39
        tax                     ; 2 41
        stx HMM0                ; 3 44
        lda #DS_MISSILE1        ; 2 46
        tay                     ; 2 48
        lda #DS_BALL            ; 2 50        
        sta RESM0               ; 3 53
        sty HMM1                ; 3 56
        jmp nk6                 ; 3 59            
        
Resm0Strobe58:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes missile size
        lda #DS_COLOR           ; 2 27
        lda #DS_HMP0            ; 2 29
        sta HMP0                ; 3 32
        lda #DS_HMP1            ; 2 34
        sta HMP1                ; 3 37
        lda #DS_MISSILE0        ; 2 39
        tax                     ; 2 41
        stx HMM0                ; 3 44
        lda #DS_BALL            ; 2 46
        sta HMBL                ; 3 49
        sta ENABL               ; 3 52 
        lda #DS_MISSILE1        ; 2 54
        sta.w RESM0             ; 4 58
        tay                     ; 2 60
        sty HMM1                ; 3 63
        lda #DS_GRP0            ; 2 65
        jmp nk7                 ; 3 68  

Resm0Strobe63:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes missile size
        lda #DS_COLOR           ; 2 27
        lda #DS_HMP0            ; 2 29
        sta HMP0                ; 3 32
        lda #DS_HMP1            ; 2 34
        sta HMP1                ; 3 37
        lda #DS_MISSILE0        ; 2 39
        tax                     ; 2 41
        stx HMM0                ; 3 44
        lda #DS_MISSILE1        ; 2 46
        tay                     ; 2 48
        sty HMM1                ; 3 51
        lda #DS_BALL            ; 2 53
        sta HMBL                ; 3 56             
        sta ENABL               ; 3 59 <- for next scanline, VDELBL on 
        sta.w RESM0             ; 4 63
        jmp nk8                 ; 3 66

Resm0Strobe68:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes missile size
        lda #DS_COLOR           ; 2 27
        lda #DS_HMP0            ; 2 29
        sta HMP0                ; 3 32
        lda #DS_HMP1            ; 2 34
        sta HMP1                ; 3 37
        lda #DS_MISSILE0        ; 2 39
        tax                     ; 2 41
        stx HMM0                ; 3 44
        lda #DS_MISSILE1        ; 2 46
        tay                     ; 2 48
        sty HMM1                ; 3 51
        lda #DS_BALL            ; 2 53
        sta HMBL                ; 3 56             
        sta ENABL               ; 3 59 <- for next scanline, VDELBL on 
        lda #DS_GRP0            ; 2 61
        sta GRP0                ; 3 64 <- for next scanline, VDELP0 on        
        sta.w RESM0             ; 4 68
        jmp nk9                 ; 3 71
        
Resm0Strobe73:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ0              ; 3 25  <- changes missile size
        lda #DS_COLOR           ; 2 27
        lda #DS_HMP0            ; 2 29
        sta HMP0                ; 3 32
        lda #DS_HMP1            ; 2 34
        sta HMP1                ; 3 37
        lda #DS_MISSILE0        ; 2 39
        tax                     ; 2 41
        stx HMM0                ; 3 44
        lda #DS_MISSILE1        ; 2 46
        tay                     ; 2 48
        sty HMM1                ; 3 51
        lda #DS_BALL            ; 2 53
        sta HMBL                ; 3 56             
        sta ENABL               ; 3 59 <- for next scanline, VDELBL on 
        lda #DS_GRP0            ; 2 61
        sta GRP0                ; 3 64 <- for next scanline, VDELP0 on        
        lda #DS_GRP1            ; 2 66 
        SLEEP 4                 ; 4 70
        sta RESM0               ; 3 73
        jmp nk10                ; 3 76/0      
        
FirstResm1:        
Resm1Strobe23:                  ;   20
        sta RESM1               ; 3 23
        lda #DS_SIZE            ; 2 25
        sta NUSIZ1              ; 3 28  <- changes missile size
        jmp nk1                 ; 3 31
        
Resm1Strobe28:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes missile size
        sta RESM1               ; 3 28
        jmp nk1                 ; 3 31
     
Resm1Strobe33:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes missile size
        lda #DS_COLOR           ; 2 27
        lda #DS_HMP0            ; 2 29
        sta.w RESM1             ; 4 33
        jmp nk2                 ; 3 36
        
Resm1Strobe38:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes missile size
        lda #DS_COLOR           ; 2 27
        lda #DS_HMP0            ; 2 29
        sta HMP0                ; 3 32
        lda #DS_HMP1            ; 2 34
        sta.w RESM1             ; 4 38
        jmp nk3                 ; 3 41
        
Resm1Strobe43:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes missile size
        lda #DS_COLOR           ; 2 27
        lda #DS_HMP0            ; 2 29
        sta HMP0                ; 3 32
        lda #DS_HMP1            ; 2 34
        sta HMP1                ; 3 37
        lda #DS_MISSILE0        ; 2 39
        sta.w RESM1             ; 4 43
        jmp nk4                 ; 3 46
        
Resm1Strobe48:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes missile size
        lda #DS_COLOR           ; 2 27
        lda #DS_HMP0            ; 2 29
        sta HMP0                ; 3 32
        lda #DS_HMP1            ; 2 34
        sta HMP1                ; 3 37
        lda #DS_MISSILE0        ; 2 39
        tax                     ; 2 41
        stx HMM0                ; 3 44
        sta.w RESM1             ; 4 48
        jmp nk5                 ; 3 51        
        
Resm1Strobe53:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes missile size
        lda #DS_COLOR           ; 2 27
        lda #DS_HMP0            ; 2 29
        sta HMP0                ; 3 32
        lda #DS_HMP1            ; 2 34
        sta HMP1                ; 3 37
        lda #DS_MISSILE0        ; 2 39
        tax                     ; 2 41
        stx HMM0                ; 3 44
        lda #DS_MISSILE1        ; 2 46
        tay                     ; 2 48
        lda #DS_BALL            ; 2 50        
        sta RESM1               ; 3 53
        sty HMM1                ; 3 56
        jmp nk6                 ; 3 59            
        
Resm1Strobe58:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes missile size
        lda #DS_COLOR           ; 2 27
        lda #DS_HMP0            ; 2 29
        sta HMP0                ; 3 32
        lda #DS_HMP1            ; 2 34
        sta HMP1                ; 3 37
        lda #DS_MISSILE0        ; 2 39
        tax                     ; 2 41
        stx HMM0                ; 3 44
        lda #DS_BALL            ; 2 46
        sta HMBL                ; 3 49
        sta ENABL               ; 3 52 
        lda #DS_MISSILE1        ; 2 54
        sta.w RESM1             ; 4 58
        tay                     ; 2 60
        sty HMM1                ; 3 63
        lda #DS_GRP0            ; 2 65
        jmp nk7                 ; 3 68  

Resm1Strobe63:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes missile size
        lda #DS_COLOR           ; 2 27
        lda #DS_HMP0            ; 2 29
        sta HMP0                ; 3 32
        lda #DS_HMP1            ; 2 34
        sta HMP1                ; 3 37
        lda #DS_MISSILE0        ; 2 39
        tax                     ; 2 41
        stx HMM0                ; 3 44
        lda #DS_MISSILE1        ; 2 46
        tay                     ; 2 48
        sty HMM1                ; 3 51
        lda #DS_BALL            ; 2 53
        sta HMBL                ; 3 56             
        sta ENABL               ; 3 59 <- for next scanline, VDELBL on 
        sta.w RESM1             ; 4 63
        jmp nk8                 ; 3 66

Resm1Strobe68:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes missile size
        lda #DS_COLOR           ; 2 27
        lda #DS_HMP0            ; 2 29
        sta HMP0                ; 3 32
        lda #DS_HMP1            ; 2 34
        sta HMP1                ; 3 37
        lda #DS_MISSILE0        ; 2 39
        tax                     ; 2 41
        stx HMM0                ; 3 44
        lda #DS_MISSILE1        ; 2 46
        tay                     ; 2 48
        sty HMM1                ; 3 51
        lda #DS_BALL            ; 2 53
        sta HMBL                ; 3 56             
        sta ENABL               ; 3 59 <- for next scanline, VDELBL on 
        lda #DS_GRP0            ; 2 61
        sta GRP0                ; 3 64 <- for next scanline, VDELP0 on        
        sta.w RESM1             ; 4 68
        jmp nk9                 ; 3 71
        
Resm1Strobe73:                  ;   20
        lda #DS_SIZE            ; 2 22
        sta NUSIZ1              ; 3 25  <- changes missile size
        lda #DS_COLOR           ; 2 27
        lda #DS_HMP0            ; 2 29
        sta HMP0                ; 3 32
        lda #DS_HMP1            ; 2 34
        sta HMP1                ; 3 37
        lda #DS_MISSILE0        ; 2 39
        tax                     ; 2 41
        stx HMM0                ; 3 44
        lda #DS_MISSILE1        ; 2 46
        tay                     ; 2 48
        sty HMM1                ; 3 51
        lda #DS_BALL            ; 2 53
        sta HMBL                ; 3 56             
        sta ENABL               ; 3 59 <- for next scanline, VDELBL on 
        lda #DS_GRP0            ; 2 61
        sta GRP0                ; 3 64 <- for next scanline, VDELP0 on        
        lda #DS_GRP1            ; 2 66 
        SLEEP 4                 ; 4 70
        sta RESM1               ; 3 73
        jmp nk10                ; 3 76/0          
    
             

MainMenuVB:   
        jsr PositionMMobjects
; PositionForLogo:
        ; ldx #3
        ; lda Frame
        ; and #1; 2 ; 1
        ; bne paOdd
        ; ldy #3          ; even frame
        ; .byte $2c
; paOdd:  ldy #7          ; odd frame
; pflLoop:
        ; lda ObjectPosition96,y
        ; jsr PosObject
        ; dey
        ; dex
        ; bpl pflLoop
        stx WSYNC
;---------------------------------------        
;        stx HMOVE           ; 3  3
        DIGITAL_AUDIO       ; 5  8
        ; ldy #%00110011      ; 2 10
        ; sty ENAM0           ; 3 13 missile on
        ; sty ENAM1           ; 3 16 missile on
        ; sty NUSIZ0          ; 3 19 three copies close, missile x8
        ; sty NUSIZ1          ; 3 22 three copies close, missile x8
        ldy #%11111111      ; 2 24
        sty GRP0            ; 3 27
        sty GRP1            ; 3 30
        ; ldy #%11110000      ; 3 32
        ; sty PF0             ; 3 35
        ; sty PF1             ; 3 38
        ; ldy #%10000000      ; 2 40
        ; sty PF2             ; 3 43
        ; sty VDELP0          ; 3 46 turn off VDEL
        ; sty VDELP1          ; 3 49 turn off VDEL
        ; sty VDELBL          ; 3 52 turn off VDEL        
;        ldy #%00000011      ; 2 54
        ldy #%00100000       ; Ball Size = 4
        sty CTRLPF          ; 2 56 reflect playfield, score mode on   
;        lda #DS_LOGO_COLORS ; 2 58 green of the station
;        sta COLUPF          ; 3 61
        ldy #BLACK          ; 3 64
        sty COLUP0          ; 3 67
        sty COLUP1          ; 3 70
        ldx #$81            ; 2 72 HMOVE_R8 & CTRLPF for score mode off
VblankWaitMenu:
        sta WSYNC           
;---------------------------------------        
        DIGITAL_AUDIO       ; 5  5        
        lda INTIM           ; 4  9
        bpl VblankWaitMenu  ; 2 11
        sty VBLANK          ; 3 14 
        ; lda Frame           ; 3 17
        ; and #1; 2 ; 1              ; 2 19
        ; beq StartEven       ; 2 21
        ; jsr SLEEP12         ;12 33
        ; jsr SLEEP12         ;12 45
        ; jsr SLEEP12         ;12 57
        ; dec Sleep5          ; 5 62
        ; dec Sleep5          ; 5 67
        ; lda #DS_LOGO_COLORS ; 2 69
        ; sta.w HMCLR         ; 4 73
        ; jmp OddLine         ; 3 76/0
        
; StartEven:        
        ; sta WSYNC        
; ;---------------------------------------                
; EvenLine:                   ;    0
        ; lda #DS_LOGO_COLORS ; 2  2
        ; sta COLUBK          ; 3  5
        ; beq LogoDone        ; 2  7 - if color was 0 we're done showing image
        ; DIGITAL_AUDIO       ; 5 12        
        ; lda #DS_LOGO_A      ; 2 14
        ; sta GRP0            ; 3 17 - must be 0-33 
        ; lda #DS_LOGO_G      ; 2 19
        ; sta GRP1            ; 3 22 - must be 0-49
        ; lda #DS_PF2         ; 2 24
        ; sta PF2             ; 3 27
        ; SLEEP 2             ; 2 29
        ; stx CTRLPF          ; 3 32 - SCORE mode off so second color shows up
        ; lda #DS_LOGO_C      ; 2 34
        ; sta GRP0            ; 3 37 - must be 36-38 
        ; lda #DS_LOGO_E      ; 2 39
        ; sta GRP0            ; 3 42 - must be 41-43        
        ; ldy #%11111100      ; 2 44
        ; sty PF1             ; 3 47 - mask 8 pixels on right that aren't hidden with missile
        ; lda #DS_LOGO_I      ; 2 49
        ; sta GRP1            ; 3 52 - must be 52-54 
        ; lda #DS_LOGO_K      ; 2 54
        ; sta GRP1            ; 3 57 - must be 57-59
        ; ldy #%00000011      ; 2 59 - D0=1 reflect playfield, D1=1 SCORE mode on
        ; sty CTRLPF          ; 3 62 - SCORE mode on to hide edges of screen
        ; stx HMP0            ; 3 65
        ; stx HMP1            ; 3 68
        ; stx HMM0            ; 3 71
        ; stx HMM1            ; 3 74
        ; lda #DS_LOGO_COLORS ; 2 76/0
; 
; OddLine:        
        ; sta HMOVE           ; 3  3 shift all objects right 8 pixels
        ; sta COLUBK          ; 3  6
        ; beq LogoDone        ; 2  8 - if color was 0 we're done showing image       
        ; DIGITAL_AUDIO       ; 5 13                
        ; lda #DS_LOGO_B      ; 2 15
        ; sta GRP0            ; 3 18 - must be 0-35
        ; lda #DS_LOGO_H      ; 2 20
        ; sta GRP1            ; 3 23 - must be 0-51
        ; lda #DS_PF2         ; 2 25
        ; sta PF2             ; 3 28
        ; ldy #%11110000      ; 2 30
        ; sty PF1             ; 3 33 - remove mask so player can draw 8 pixels on right        
        ; stx CTRLPF          ; 3 36 - SCORE mode off so second color shows up
        ; lda #DS_LOGO_D      ; 2 38
        ; sta GRP0            ; 3 41 - must be 39-41
        ; lda #DS_LOGO_F      ; 2 43
        ; sta GRP0            ; 3 46 - must be 44-46
        ; ldy #%00000011      ; 2 48 D0=1 reflect playfield, D1=1 SCORE mode on
        ; lda #DS_LOGO_J      ; 2 50
        ; SLEEP 2             ; 2 52
        ; sta GRP1            ; 3 55 - must be 55-57
        ; lda #DS_LOGO_L      ; 2 57
        ; sta GRP1            ; 3 60 - must be 60-62
        ; sty CTRLPF          ; 3 63 - SCORE mode on to hide edges of screen
        ; SLEEP 4             ; 4 67
        ; sta HMCLR           ; 3 70 - 27-70
        ; sta HMOVE           ; 3 73 - shift all objects left 8 pixels
        ; jmp EvenLine        ; 3 76/0
        ; 
        ; ; NEED TO REVIEW THIS
; LogoDone:                   ;    9 (worse case from OddLine)
        ; DIGITAL_AUDIO       ; 5 14  
        ; ldy #0              ; 2 16
        ; sty COLUPF          ; 3 19 
        ; sty GRP0            ; 3 22
        ; sty GRP1            ; 3 25
        ; sty ENAM0           ; 3 28
        ; sty ENAM1           ; 3 31
        ; sty PF0             ; 3 34
        ; sty PF1             ; 3 37
        ; sty PF2             ; 3 40
        ; ldy #%00110011      ; 2 42
        ; sty ENAM0           ; 3 45
        ; sty ENAM1           ; 3 48
        ; sty ENABL           ; 3 51
        ; sty NUSIZ0          ; 3 54
        ; sty NUSIZ1          ; 3 57
        ; sty VDELP0          ; 3 60
        ; sty VDELP1          ; 3 63
        ; ldy #%00100000      ; 2 65 Ball Size = 4
        ; sty CTRLPF          ; 3 68
        ; ; END NEED TO REVIEW THIS
        ; 
        ; ldy #MM_TITLE_GAP   ; 2 69
; tgLoop:
        ; sty WSYNC
        ; DIGITAL_AUDIO       ; 5  5          
        ; dey                 ; 2  7
        ; bne tgLoop          ; 2  9
        ; 
        ; jsr PositionMMobjects
        ; 
        ; ldx #0   
        ; stx VDELBL  ; not needed?
        
MMloop:        
        lda #<DS_MENU_CONTROL
        sta PF0
        lda #<DS_MENU_CONTROL
        sta PF1
        lda #<DS_MENU_CONTROL
        sta PF2
        sta HMCLR
        ldx #4
        lda #<DS_MENU_CONTROL
        jsr PosObject
        sta WSYNC
        sta HMOVE
        DIGITAL_AUDIO       ; 5  5        
        lda #<DS_MENU_CONTROL
        tay
        jsr ShowTwoColorGraphic
                                ;   19
        lda #<DS_MENU_CONTROL
        bmi MMdone
        beq MMloop
        tay
MMgap:
        sta WSYNC
        dey
        beq MMloop
        bne MMgap
       
MMdone:
        sta WSYNC
        stx ENAM0    ; X=0 after ShowTwoColorGraphic
        stx ENAM1
        stx ENABL
        
        ldx #MM_LOGO_GAP
SLgap:
        sta WSYNC
        dex
        bpl SLgap

ShowSpiceWareLogo:  
        ldx #WHITE
        stx COLUP0
        stx COLUP1
        stx COLUPF
    
        ldy #7
        sty LoopCounter
        ; show 1st line
        sta WSYNC
;---------------------------------------        
        DIGITAL_AUDIO       ; 5  5
        lda #DS_MENU_A      ; 2  7
        sta GRP0            ; 3 10
        lda #DS_MENU_A      ; 2 12
        sta GRP1            ; 3 15
        lda #DS_MENU_A      ; 2 17
        sta GRP0            ; 3 20
        lda #DS_MENU_A      ; 2 22
        tax                 ; 2 24
        lda #DS_MENU_A      ; 2 26
        tay                 ; 2 28
        lda #DS_MENU_A      ; 2 30
        SLEEP 11            ;11 41
        stx GRP1            ; 3 44 <- time critical 
        sty GRP0            ; 3 47
        sta GRP1            ; 3 50
        sta GRP0            ; 3 53
        sty PF1             ; 3 56  ; Y has $FF from extended line in SW logo
        sty PF2             ; 3 59
        lda #DS_MENU_A      ; 2 61  ; next line
        sta GRP0            ; 3 64
        lda #DS_MENU_A      ; 2 66
        sta GRP1            ; 3 69
        iny                 ; 2 71
        sty PF1             ; 3 74
        sty PF2             ; 3 77/1
        DIGITAL_AUDIO       ; 5  6 space for audio if we need it
        SLEEP 4             ; 4 10
        tya                 ; 2 12    
        beq SSWLskip1       ; 3 15
    
SSWL1:
        sta WSYNC
;---------------------------------------        
        DIGITAL_AUDIO       ; 5  5
        lda #DS_MENU_A      ; 2  7
        sta GRP0            ; 3 10
        lda #DS_MENU_A      ; 2 12
        sta GRP1            ; 3 15
SSWLskip1:
        lda #DS_MENU_A      ; 2 17
        sta GRP0            ; 3 20
        lda #DS_MENU_A      ; 2 22
        tax                 ; 2 24
        lda #DS_MENU_A      ; 2 26
        tay                 ; 2 28
        lda #DS_MENU_A      ; 2 30
        SLEEP 6             ; 6 36  
        dec LoopCounter     ; 5 41
        stx GRP1            ; 3 44
        sty GRP0            ; 3 47
        sta GRP1            ; 3 50
        sta GRP0            ; 3 53
        bne SSWL1           ; 3 56  
                            ; 2 55 if not taken
                            ; show last line
        lda #DS_MENU_A      ; 2 57
        tax
        stx GRP0            ; 3 60
        lda #DS_MENU_A      ; 2 62
        sta GRP1            ; 3 65
        stx PF0             ; 3 68  ; X = $FF
        stx PF1             ; 3 71
        ldx #%00000011      ; 2 73
;---------------------------------------
        DIGITAL_AUDIO       ; 5  78/4
        stx ENAM0           ; 3  7
        ldx #1              ; 2  9
        stx CTRLPF          ; 3 12  ; reflect playfield
        lda #DS_MENU_A      ; 2 14
        sta GRP0            ; 3 17
        lda #DS_MENU_A      ; 2 19
        tax                 ; 2 21
        lda #DS_MENU_A      ; 2 23
        tay                 ; 2 25
        lda #DS_MENU_A      ; 2 27
        SLEEP 14            ;14 41    
        stx GRP1            ; 3 44 <- time critical 
        sty GRP0            ; 3 47
        sta GRP1            ; 3 50
        sta GRP0            ; 3 53
        ldy #0              ; 2 55
        sty PF1             ; 3 58
        sty PF0             ; 3 61
        sty ENAM0           ; 3 64
        sty GRP1            ; 3 67
        sty GRP0            ; 3 70
        sty GRP1            ; 3 73
EndShowSpiceWareLogo:           

        
MenuKernelDone:
        stx WSYNC
        DIGITAL_AUDIO       ; 5  5          
        ldx #2
        stx VBLANK  ; Video Blank ON (thus image output off)
        ldx #33+1
        stx TIM64T
        ldx #RUN_MENU_OS                
        jmp OverScan
        
        
        
SplashVB:      ; splash routines 55 bytes
        jsr PositionMMobjects
        
        ldy #%11000000      ; 2
        sty PF0             ; 3
        sty PF1             ; 3
        
        ; ball at 89...
        
        ldx #0
VblankWaitSplash        
        sta WSYNC
        lda INTIM
        bpl VblankWaitSplash        ; 2  6
        stx VBLANK 
        
        ldy #80
SplashKernelTop:
        sty WSYNC
        dey
        bne SplashKernelTop
        
        ldy #25
        jsr ShowTwoColorGraphic
        
        ldy #81
SplashKernelBottom:
        sty WSYNC
        dey
        bne SplashKernelBottom

SplashKernelDone:
        stx WSYNC
        ldx #2
        stx VBLANK  ; Video Blank ON (thus image output off)
        ldx #33+1
        stx TIM64T
        ldx #RUN_SPLASH_OS        
        jmp OverScan   
    
 
ObjectPosition96: ; thru rts = 46 bytes
        ; even frame
        .byte 32    ; P0 - left side of 96 pixel display
        .byte 80    ; P1 - right side of 96 pixel display
        .byte 24+1  ; M0 - hide P0 gaps on left side - missiles are 1 off from players
        .byte 72+1  ; M0 - hide P1 gaps on right side    
        ; odd frame        
        .byte 40    ; P0 - left side of 96 pixel display
        .byte 88    ; P1 - right side of 96 pixel display
        .byte 32+1  ; M0 - hide P0 gaps on left side
        .byte 80+1  ; M0 - hide P1 gaps on right side    
        
MMobjectPosition:
        .byte $38   ; P0 for menu
        .byte $40   ; P1 for menu
        .byte 17    ; Missile 0 - to hide MM logo garbage
        .byte 105   ; Missile 1 - to hide MM logo garbage    
        
PositionMMobjects:
        ldx #3   
pmmoLoop:   
        lda MMobjectPosition,x
        jsr PosObject
        dex
        bpl pmmoLoop
        sta WSYNC
        sta HMOVE  
        DIGITAL_AUDIO
        ldy #%00110011      ; 2 
        sty ENAM0           ; 3 missile on
        sty ENAM1           ; 3 missile on
        sty NUSIZ0          ; 3 three copies close, missile x8
        sty NUSIZ1          ; 3 three copies close, missile x8
        sty VDELP0          ; 3 
        sty VDELP1          ; 3         
        rts                        
        
    align 2 ; Samples thru Rates = 30 bytes
Samples = [* & $fff] + $7000
    .word sample_blastoff
    .word sample_alert
    .word sample_battlestations
    .word sample_spyship
    .word sample_conditionred
    
SampleSizes = [* & $fff] + $7000
    .word sample_blastoff_size
    .word sample_alert_size
    .word sample_battlestations_size
    .word sample_spyship_size
    .word sample_conditionred_size

SampleRates = [* & $fff] + $7000
    .word 110000 / 256  ; blastoff
    .word  94000 / 256  ; alert
    .word  95000 / 256  ; battlestations
    .word  98000 / 256  ; spy ship
    .word  92000 / 256  ; condition red
        

        include "stationary_data_bank6.asm" ' ; 293 bytes
        include "sfx.asm"   ; 624 bytes
 
        echo "----", [$FFF0 - *]d, "bytes of Bank 6 left"         

    ; ROM Start Pointers
    ORG $7FF0
    RORG $FFF0
    DC.B $00 ; 0 DSWRITE
    DC.B $00 ; 1 DSPTR
    DC.B $00 ; 2 SETMODE
    DC.B $00 ; 3 CALLFN
    DC.B $00 ; 4
SelectBank0:    DC.B $00 ; 5
SelectBank1:    DC.B $00 ; 6
SelectBank2:    DC.B $00 ; 7
SelectBank3:    DC.B $00 ; 8
SelectBank4:    DC.B $00 ; 9
SelectBank5:    DC.B $00 ; a
SelectBank6:    DC.B $00 ; b
    DC.W Start ; cd
    DC.W Start ; ef   
    
;===============================================================================
; Defines for C
;-------------------------------------------------------------------------------
; These echo statements are used to create the header file defines_dasm.h
; which is used by the C code to reference information in the 6507 source
;===============================================================================
    echo "#define DD_BASE 0x40000800"
    echo ""
    echo "// data from 6507 to ARM"
    ; echo "#define FUNC          (*(unsigned char *)( DD_BASE +",[ARMfunc]d,"))"
    ; echo "#define SWCHA         (*(unsigned char *)( DD_BASE +",[ARMswcha]d,"))"
    ; echo "#define SWCHB         (*(unsigned char *)( DD_BASE +",[ARMswchb]d,"))"
    ; echo "#define INPT4         (*(unsigned char *)( DD_BASE +",[ARMinpt4]d,"))"
    ; echo "#define VB_TIME       (*(unsigned char *)( DD_BASE +",[ARMvbtime]d,"))"
    ; echo "#define OS_TIME       (*(unsigned char *)( DD_BASE +",[ARMostime]d,"))"
    echo "#define FUNC          QUEUE[",[ARMfunc]d,"]"
    echo "#define SWCHA         QUEUE[",[ARMswcha]d,"]"
    echo "#define SWCHB         QUEUE[",[ARMswchb]d,"]"
    echo "#define INPT4         QUEUE[",[ARMinpt4]d,"]"
    echo "#define VB_TIME       QUEUE[",[ARMvbtime]d,"]"
    echo "#define OS_TIME       QUEUE[",[ARMostime]d,"]"
    echo ""
    echo "// data from ARM to 6507 - from overscan call"
    ; echo "#define MODE          (*(unsigned char *)( DD_BASE +",[ARMmode]d,"))"
    ; echo "#define FRAME         (*(unsigned char *)( DD_BASE +",[ARMframe]d,"))"
    ; echo "#define AUDV          ((unsigned char *)( DD_BASE +",[ARMaudv0]d,"))"
    ; echo "#define AUDF          ((unsigned char *)( DD_BASE +",[ARMaudf0]d,"))"
    ; echo "#define AUDC          ((unsigned char *)( DD_BASE +",[ARMaudc0]d,"))"        
    ; echo "#define AUDV0         (*(unsigned char *)( DD_BASE +",[ARMaudv0]d,"))"
    ; echo "#define AUDF0         (*(unsigned char *)( DD_BASE +",[ARMaudf0]d,"))"
    ; echo "#define AUDC0         (*(unsigned char *)( DD_BASE +",[ARMaudc0]d,"))"
    ; echo "#define AUDV1         (*(unsigned char *)( DD_BASE +",[ARMaudv1]d,"))"
    ; echo "#define AUDF1         (*(unsigned char *)( DD_BASE +",[ARMaudf1]d,"))"
    ; echo "#define AUDC1         (*(unsigned char *)( DD_BASE +",[ARMaudc1]d,"))"
    echo "#define MODE          QUEUE[",[ARMmode]d,"]"
    echo "#define FRAME         QUEUE[",[ARMframe]d,"]"
    echo "#define AUDV          ((unsigned char *)( DD_BASE +",[ARMaudv0]d,"))"
    echo "#define AUDF          ((unsigned char *)( DD_BASE +",[ARMaudf0]d,"))"
    echo "#define AUDC          ((unsigned char *)( DD_BASE +",[ARMaudc0]d,"))"        
    echo "#define AUDV0         QUEUE[",[ARMaudv0]d,"]"
    echo "#define AUDF0         QUEUE[",[ARMaudf0]d,"]"
    echo "#define AUDC0         QUEUE[",[ARMaudc0]d,"]"
    echo "#define AUDV1         QUEUE[",[ARMaudv1]d,"]"
    echo "#define AUDF1         QUEUE[",[ARMaudf1]d,"]"
    echo "#define AUDC1         QUEUE[",[ARMaudc1]d,"]"
    echo "#define AUDIO_QUIET   ",[ARMaudc0]d
        
    echo ""
    echo "// constants used to specify which function to run"
    echo "#define RUN_INIT       ",[RUN_INIT]d
    echo "#define RUN_MENU_VB    ",[RUN_MENU_VB]d
    echo "#define RUN_MENU_OS    ",[RUN_MENU_OS]d
    echo "#define RUN_GAME_VB    ",[RUN_GAME_VB]d
    echo "#define RUN_GAME_OS    ",[RUN_GAME_OS]d
    echo "#define RUN_SPLASH_VB  ",[RUN_SPLASH_VB]d
    echo "#define RUN_SPLASH_OS  ",[RUN_SPLASH_OS]d
    echo "#define RUN_INIT_LEVEL ",[RUN_INIT_LEVEL]d
    echo ""
    echo "// datastreams"
    ; echo "#define DS_LOGO_A         ",[DS_LOGO_A]d
    ; echo "#define DS_LOGO_B         ",[DS_LOGO_B]d
    ; echo "#define DS_LOGO_C         ",[DS_LOGO_C]d
    ; echo "#define DS_LOGO_D         ",[DS_LOGO_D]d
    ; echo "#define DS_LOGO_E         ",[DS_LOGO_E]d
    ; echo "#define DS_LOGO_F         ",[DS_LOGO_F]d
    ; echo "#define DS_LOGO_G         ",[DS_LOGO_G]d
    ; echo "#define DS_LOGO_H         ",[DS_LOGO_H]d
    ; echo "#define DS_LOGO_I         ",[DS_LOGO_I]d
    ; echo "#define DS_LOGO_J         ",[DS_LOGO_J]d
    ; echo "#define DS_LOGO_K         ",[DS_LOGO_K]d
    ; echo "#define DS_LOGO_L         ",[DS_LOGO_L]d
    ; echo "#define DS_PF2            ",[DS_PF2]d
    ; echo "#define DS_LOGO_COLORS    ",[DS_LOGO_COLORS]d
    echo "#define DS_MENU_CONTROL   ",[DS_MENU_CONTROL]d
    echo "#define DS_MENU_A         ",[DS_MENU_A]d
    echo "#define DS_MENU_COLOR     ",[DS_MENU_COLOR]d
    echo "#define DS_GRP0           ",[DS_GRP0]d
    echo "#define DS_GRP1           ",[DS_GRP1]d
    echo "#define DS_HMP0           ",[DS_HMP0]d
    echo "#define DS_HMP1           ",[DS_HMP1]d
    echo "#define DS_MISSILE0       ",[DS_MISSILE0]d
    echo "#define DS_MISSILE1       ",[DS_MISSILE1]d
    echo "#define DS_BALL           ",[DS_BALL]d
    echo "#define DS_COLOR          ",[DS_COLOR]d
    echo "#define DS_SIZE           ",[DS_SIZE]d
    echo "#define DS_RADAR_CONTROL  ",[DS_RADAR_CONTROL]d
    echo "#define DS_RADAR_GRP0     ",[DS_RADAR_GRP0]d
    echo "#define DS_RADAR_GRP1     ",[DS_RADAR_GRP1]d
    echo "#define DS_FORMATION_GRP0 ",[DS_FORMATION_GRP0]d
    echo "#define DS_FORMATION_GRP1 ",[DS_FORMATION_GRP1]d
    echo "#define DS_PF0L           ",[DS_PF0L]d
    echo "#define DS_PF1L           ",[DS_PF1L]d
    echo "#define DS_PF2L           ",[DS_PF2L]d
    echo "#define DS_PF0R           ",[DS_PF0R]d
    echo "#define DS_PF1R           ",[DS_PF1R]d
    echo "#define DS_PF2R           ",[DS_PF2R]d
    echo "#define DS_STATION_COLOR  ",[DS_STATION_COLOR]d
    echo "#define SCORE_COLOR       ",[SCORE_COLOR]d
    
    echo ""
    echo "// Splash Screen"
    echo "#define SPLASH_SCREEN_GRAPHICS_ROM ", [SplashScreenGraphicsROM]d
    echo "#define SPLASH_SCREEN_GFX_SIZE     ", [SplashScreenGraphicsSize]d
    echo ""
    
    echo "// Main Menu"
    echo "#define MAIN_MENU_GRAPHICS_ROM     ", [MainMenuGraphicsROM]d
    echo "#define MAIN_MENU_GFX_SIZE         ", [MainMenuGraphicsSize]d
    echo "#define MAIN_MENU_GRAPHICS_RAM     ", [MainMenuGraphicsRAM]d
    echo "#define LOGO_HEIGHT                 90"
    echo "#define MAIN_MENU_COLORS_OFFSET    ", [MainMenuColors-MainMenuGraphicsROM]d
    echo "#define MAIN_MENU_OPTIONS_OFFSET   ", [MainMenuOptions-MainMenuGraphicsROM]d
    echo "#define MAIN_MENU_CONTROL_OFFSET   ", [MenuControl-MainMenuGraphicsROM]d
    echo "#define MENU_DIGITS                ", [MenuDigits]d
    echo "#define TITLE_SCREEN_COLOR_OFFSET  ", [TitlescreenColors-MainMenuGraphicsROM]d
    echo "#define TITLE_SCREEN_COLOR_SIZE    ", [TitlescreenColorsSize]d
    echo "#define START_COLOR_OFFSET         ", [MenuStartColor-MainMenuGraphicsROM]d
    echo "#define MENU_GRAPHICS_OFFSET       ((unsigned short int *)(",[MenuOptionGfx-2]d,"))"
    echo "#define MENU_FIRST_OPTION_ID       ", [MENU_FIRST_OPTION_ID]d
    echo "#define MENU_QUADRANT_ID           ", [MENU_QUADRANT_ID]d
    echo "#define MENU_SECTOR_ID             ", [MENU_SECTOR_ID]d
    echo "#define MENU_LEVEL_ID              ", [MENU_LEVEL_ID]d
    echo "#define MENU_TV_TYPE_ID            ", [MENU_TV_TYPE_ID]d
    echo "#define MENU_START_ID              ", [MENU_START_ID]d
    echo "#define MENU_LAST_ID               ", [MENU_LAST_ID]d
    echo "#define MENU_HIGH_ID               ", [MENU_HIGH_ID]d
    ; echo "#define MM_JOYSTICK_TIMER          (*(unsigned char *)(DD_BASE +",[MMjoystick]d,"))"
    ; echo "#define MM_FIRE_DOWN               (*(unsigned char *)(DD_BASE +",[MMfire]d,"))"
    ; echo "#define MM_SELECTED_OPTION         (*(unsigned char *)(DD_BASE +",[MMselected]d,"))"
    ; echo "#define MM_QUADRANT                (*(unsigned char *)(DD_BASE +",[MM_Quadrant]d,"))"
    ; echo "#define MM_INITIAL_SECTOR          (*(unsigned char *)(DD_BASE +",[MM_InitialSector]d,"))"
    ; echo "#define MM_LEVEL                   (*(unsigned char *)(DD_BASE +",[MM_Level]d,"))"
    ; echo "#define MM_TV_TYPE                 (*(unsigned char *)(DD_BASE +",[MM_TVtype]d,"))"
    echo "#define MM_JOYSTICK_TIMER          QUEUE[",[MMjoystick]d,"]"
    echo "#define MM_FIRE_DOWN               QUEUE[",[MMfire]d,"]"
    echo "#define MM_SELECTED_OPTION         QUEUE[",[MMselected]d,"]"
    echo "#define MM_QUADRANT                QUEUE[",[MM_Quadrant]d,"]"
    echo "#define MM_INITIAL_SECTOR          QUEUE[",[MM_InitialSector]d,"]"
    echo "#define MM_LEVEL                   QUEUE[",[MM_Level]d,"]"
    echo "#define MM_TV_TYPE                 QUEUE[",[MM_TVtype]d,"]"
    echo "#define OPTION_BETA                ", [OptionBeta]d
    echo "#define OPTION_GAMMA               ", [OptionGamma]d
    echo "#define OPTION_DELTA               ", [OptionDelta]d    
    echo "#define OPTION_NORMAL              ", [OptionNormal]d
    echo "#define OPTION_EASY                ", [OptionEasy]d
    echo "#define OPTION_HARD                ", [OptionHard]d
    echo "#define OPTION_PAL                 ", [OptionPAL]d
    echo "#define OPTION_SECAM               ", [OptionSECAM]d
    echo ""
    echo "// Sound Effects & Tunes (which are concurrent sound effects)"
    echo "#define SOUND_FREQ                 ((unsigned char *)(",[[SoundFreq & $fff] + $7000]d,"))"
    echo "#define SOUND_VOL                  ((unsigned char *)(",[[SoundControlVolume & $fff] + $7000]d,"))"
    ; these are finished sounds
    echo "#define SOUND_PLAYER_SHOOTS_ID     ",[SoundPlayerShoot]d
    echo "#define SOUND_CANNON_HIT_ID        ",[_05_CannonHit]d
    echo "#define SOUND_ITYPE_SPY_HIT_ID     ",[_08_MissileSpyShipHit]d
    echo "#define TUNE_ROUND_CLEAR_A         ",[_02_RoundClearA]d
    echo "#define TUNE_ROUND_CLEAR_B         ",[_02_RoundClearB]d
    echo "#define SOUND_ETYPE_HIT_ID         ",[_09_E_Type_MissileHit]d
    echo "#define SOUND_PTYPE_HIT_ID         ",[_06_P_Type]d
    echo "#define SOUND_ASTEROID_ID          ",[_11_AsteroidHit]d
    echo "#define SOUND_FORMATION_ID         ",[_07_EnemyFormationHit]d
    echo "#define SOUND_CONDITION_RED_ID     ",[_10TenSndAlertCondRed]d
    echo "#define SOUND_CONDITION_RED_LOOP   ",[_10ActiveAlertCondRed]d
    echo "#define SOUND_MENU_BLIP_ID         ",[SoundMenuBlip]d
    ; these are placeholder sounds
    echo "#define SOUND_MINE_ID              ",[SoundMineHit]d
    echo "#define SOUND_PLAYER_HIT_ID        ",[SoundPlayerHit]d
    
    ; SongEnd is really for getting a high score
    echo "#define TUNE_NEW_GAME_A            ",[_01_SongEndA]d ; [SongBeginA]d
    echo "#define TUNE_NEW_GAME_B            ",[_01_SongEndB]d ; [SongBeginB]d
    echo "#define TUNE_HIGH_SCORE_A          ",[_01_SongEndA]d ; [SongBeginA]d
    echo "#define TUNE_HIGH_SCORE_B          ",[_01_SongEndB]d ; [SongBeginB]d
    ;echo "#define TUNE_MENU_BLIP_A           ",[_03_TrampolineA]d
    ;echo "#define TUNE_MENU_BLIP_B           ",[_03_TrampolineB]d
    
    echo "#define SAMPLES                    ((unsigned short int *)(",[Samples]d,"))"
    echo "#define SAMPLE_SIZES               ((unsigned short int *)(",[SampleSizes]d,"))"
    echo "#define SAMPLE_RATES               ((unsigned short int *)(",[SampleRates]d,"))"
    echo "#define SAMPLE_BLAST_OFF           1"
    echo "#define SAMPLE_ALERT_ALERT         2"
    echo "#define SAMPLE_BATTLESTATIONS      3"
    echo "#define SAMPLE_SPY_SHIP_SIGHTED    4"
    echo "#define SAMPLE_CONDITION_RED       5"
    echo ""
    echo "// game screen datastreams"
    echo "#define DATASTREAM_SIZE            ",[DATASTREAM_SIZE]d
    echo "#define REPOSITION_SIZE            ",[REPOSITION_SIZE]d
    echo "#define CLEAR_DS_INT               ",[ClearDatastreamInt]d    
    echo "#define CLEAR_DS_INT_SIZE          ",[ClearDatastreamIntSize/4]d
    echo "#define P0_DATASTREAM              ",[Player0DataStream]d
    echo "#define P1_DATASTREAM              ",[Player1DataStream]d
    echo "#define HMP0_DATASTREAM            ",[HMP0DataStream]d
    echo "#define HMP1_DATASTREAM            ",[HMP1DataStream]d
    echo "#define MISSILE0_DATASTREAM        ",[Missile0DataStream]d
    echo "#define MISSILE1_DATASTREAM        ",[Missile1DataStream]d
    echo "#define BALL_DATASTREAM            ",[BallDataStream]d
    echo "#define COLOR_DATASTREAM           ",[ColorDataStream]d
    echo "#define SIZE_DATASTREAM            ",[SizeDataStream]d
    echo "#define JUMP_DATASTREAM            ",[JumpDataStream]d
    echo "#define SCORE_DATASTREAM           ",[ScoreDataStream]d
    echo "#define EXIT_JUMP_DS               (*(unsigned short int *)(DD_BASE +",[ExitJumpDataStream]d,"))"
    echo "#define EXIT_SCORE_DS              (*(unsigned short int *)(DD_BASE +",[ExitScoreDataStream]d,"))"
;    echo "#define EXIT_JUMP_DS               QUEUE_SINT[",[ExitJumpDataStream]d,"]"
;    echo "#define EXIT_SCORE_DS              QUEUE_SINT[",[ExitScoreDataStream]d,"]"
    echo ""
    echo "// score, radar, status"
    echo "#define RADAR_HEIGHT               ", [RADAR_HEIGHT]d
    echo "#define RADAR_P0                   ", [RadarP0DataStream]d
    echo "#define RADAR_P1                   ", [RadarP1DataStream]d
    echo "#define FORMATION_P0               ", [FormationP0DataStream]d
    echo "#define FORMATION_P1               ", [FormationP1DataStream]d
    echo "#define PLAYFIELD_DATA             ", [PF0L]d
    echo "#define PF0L                       ", [PF0L-PF0L]d
    echo "#define PF1L                       ", [PF1L-PF0L]d
    echo "#define PF2L                       ", [PF2L-PF0L]d
    echo "#define PF0R                       ", [PF0R-PF0L]d
    echo "#define PF1R                       ", [PF1R-PF0L]d
    echo "#define PF2R                       ", [PF2R-PF0L]d
    echo "#define PF_FONT                    ((unsigned char *)(",[PFfont]d,"))"
    echo "#define PF_FONT_HEIGHT             ", [PFfontHeight]d
    echo "#define RADAR_CONTROL              ", [RadarControl]d
    echo ""
    echo "// kernels"
    echo "#define NORMAL_KERNEL              ",[NormalKernel]d
    echo "#define EXIT_KERNEL                ",[ExitKernel]d
    echo "#define INIT_LEVEL_KERNEL          ",[InitLevelKernel]d
    echo "#define FIRST_RESP0                ",[FirstResp0]d
    echo "#define FIRST_RESP1                ",[FirstResp1]d
    echo "#define FIRST_RESM0                ",[FirstResm0]d
    echo "#define FIRST_RESM1                ",[FirstResm1]d
    echo "#define FIRST_RESBL                ",[FirstResbl]d
    echo "#define RESP0STROBE23              ",[Resp0Strobe23]d
    echo "#define RESP0STROBE28              ",[Resp0Strobe28]d
    echo "#define RESP0STROBE33              ",[Resp0Strobe33]d
    echo "#define RESP0STROBE38              ",[Resp0Strobe38]d
    echo "#define RESP0STROBE43              ",[Resp0Strobe43]d
    echo "#define RESP0STROBE48              ",[Resp0Strobe48]d
    echo "#define RESP0STROBE53              ",[Resp0Strobe53]d
    echo "#define RESP0STROBE58              ",[Resp0Strobe58]d
    echo "#define RESP0STROBE63              ",[Resp0Strobe63]d
    echo "#define RESP0STROBE68              ",[Resp0Strobe68]d
    echo "#define RESP0STROBE73              ",[Resp0Strobe73]d
    echo "#define RESP1STROBE23              ",[Resp1Strobe23]d
    echo "#define RESP1STROBE28              ",[Resp1Strobe28]d
    echo "#define RESP1STROBE33              ",[Resp1Strobe33]d
    echo "#define RESP1STROBE38              ",[Resp1Strobe38]d
    echo "#define RESP1STROBE43              ",[Resp1Strobe43]d
    echo "#define RESP1STROBE48              ",[Resp1Strobe48]d
    echo "#define RESP1STROBE53              ",[Resp1Strobe53]d
    echo "#define RESP1STROBE58              ",[Resp1Strobe58]d
    echo "#define RESP1STROBE63              ",[Resp1Strobe63]d
    echo "#define RESP1STROBE68              ",[Resp1Strobe68]d
    echo "#define RESP1STROBE73              ",[Resp1Strobe73]d
    echo "#define RESM0STROBE23              ",[Resm0Strobe23]d
    echo "#define RESM0STROBE28              ",[Resm0Strobe28]d
    echo "#define RESM0STROBE33              ",[Resm0Strobe33]d
    echo "#define RESM0STROBE38              ",[Resm0Strobe38]d
    echo "#define RESM0STROBE43              ",[Resm0Strobe43]d
    echo "#define RESM0STROBE48              ",[Resm0Strobe48]d
    echo "#define RESM0STROBE53              ",[Resm0Strobe53]d
    echo "#define RESM0STROBE58              ",[Resm0Strobe58]d
    echo "#define RESM0STROBE63              ",[Resm0Strobe63]d
    echo "#define RESM0STROBE68              ",[Resm0Strobe68]d
    echo "#define RESM0STROBE73              ",[Resm0Strobe73]d
    echo "#define RESM1STROBE23              ",[Resm1Strobe23]d
    echo "#define RESM1STROBE28              ",[Resm1Strobe28]d
    echo "#define RESM1STROBE33              ",[Resm1Strobe33]d
    echo "#define RESM1STROBE38              ",[Resm1Strobe38]d
    echo "#define RESM1STROBE43              ",[Resm1Strobe43]d
    echo "#define RESM1STROBE48              ",[Resm1Strobe48]d
    echo "#define RESM1STROBE53              ",[Resm1Strobe53]d
    echo "#define RESM1STROBE58              ",[Resm1Strobe58]d
    echo "#define RESM1STROBE63              ",[Resm1Strobe63]d
    echo "#define RESM1STROBE68              ",[Resm1Strobe68]d
    echo "#define RESM1STROBE73              ",[Resm1Strobe73]d
    echo "#define RESBLSTROBE23              ",[ResblStrobe23]d
    echo "#define RESBLSTROBE28              ",[ResblStrobe28]d
    echo "#define RESBLSTROBE33              ",[ResblStrobe33]d
    echo "#define RESBLSTROBE38              ",[ResblStrobe38]d
    echo "#define RESBLSTROBE43              ",[ResblStrobe43]d
    echo "#define RESBLSTROBE48              ",[ResblStrobe48]d
    echo "#define RESBLSTROBE53              ",[ResblStrobe53]d
    echo "#define RESBLSTROBE58              ",[ResblStrobe58]d
    echo "#define RESBLSTROBE63              ",[ResblStrobe63]d
    echo "#define RESBLSTROBE68              ",[ResblStrobe68]d
    echo "#define RESBLSTROBE73              ",[ResblStrobe73]d
    echo "#define SCORE_KERNEL               ",[ScoreKernel]d
    echo "#define EXIT_SCORE_KERNEL          ",[ExitScoreKernel]d  
    echo ""
    echo "// Image Data"
    echo "#define IMAGE_HEIGHTS              ", [ImageHeights]d
    echo "#define IMAGE_SHIFT                ", [ImageShift]d
    echo "#define IMAGE_GRAPHICS             ", [ImageGraphics]d
    echo "#define MISSILE_GRAPHICS           ", [MissileGraphics]d
    echo "#define STATION_IMAGES             ", [StationImages]d
    echo "#define STATION_HEIGHT             ", [StationHorizontalHeight+1]d    
    echo "#define OBJECT_COLORS_NTSC         ", [ObjectColorsNTSC]d
    echo "#define OBJECT_COLORS_SIZE         ", [ObjectColorSize]d
    echo "#define OBJECT_COLORS_RAM          ", [ObjectColors]d
    echo ""
    echo "// Image IDs"
    echo "#define SHIFTING_SPRITES           ", [ShiftingSprites]d
    echo "#define STATION_HORIZONTAL_ID      ", [StationHorizontalID]d
    echo "#define STATION_VERTICAL_ID        ", [StationVerticalID]d
    echo "#define ASTEROID_ID                ", [AsteroidID]d
    echo "#define MINE_ID                    ", [MineID]d
    echo "#define SHIP_ID                    ", [ShipID]d
    echo "#define SHIP_HEIGHT                ", [Ship0Height]d
    echo "#define ETYPE_ID                   ", [ETypeID]d
    echo "#define ITYPE_ID                   ", [ITypeID]d
    echo "#define PTYPE_ID                   ", [PTypeID]d
    echo "#define SPY_ID                     ", [SpyID]d
    echo "#define POD_ID                     ", [PodID]d
    echo "#define CORE_HORIZONTAL_ID         ", [EnergyPodHorizontalCollisionID]d
    echo "#define CORE_VERTICAL_ID           ", [EnergyPod_VerticalCollisionID]d
    echo "#define FORMATION_ID               ", [FormationID]d    
    echo "#define FIRST_MISSILE_GRAPHIC_ID   ", [FirstMissileGraphicID]d
    echo "#define SHOT_HORIZONTAL_ID         ", [ShotHorizontalID - FirstMissileGraphicID]d
    echo "#define SHOT_DIAGONAL_TLBR_ID      ", [ShotDiagonal_TLBRID - FirstMissileGraphicID]d
    echo "#define SHOT_DIAGONAL_TRBL_ID      ", [ShotDiagonal_TRBLID - FirstMissileGraphicID]d
    echo "#define SHOT_VERTICAL_ID           ", [ShotVerticalID - FirstMissileGraphicID]d
    echo "#define SHOT_STATION_ID            ", [ShotStationID - FirstMissileGraphicID]d
    echo "#define ENERGY_POD_HORIZONTAL_ID   ", [EnergyPodHorizontalID - FirstMissileGraphicID]d
    echo "#define ENERGY_POD_VERTICAL_ID     ", [EnergyPodVerticalID - FirstMissileGraphicID]d
    echo "#define HORIZONTAL_POD_DIFF_ID     ", [HorizontalPodID]d
    echo "#define VERTICAL_POD_DIFF_ID       ", [VerticalPodID]d
    echo "#define EXPLOSION_PLAYER_ID        ", [ExplosionPlayerID]d
    echo "#define EXPLOSION_MINE_ID          ", [ExplosionMineID]d
    echo "#define EXPLOSION_STATION_ID       ", [ExplosionStationID]d
    echo "#define EXPLOSION_ENEMYSHIP_ID     ", [ExplosionEnemyShipID]d
    echo "#define EXPLOSION_ASTEROIDPOD_ID   ", [ExplosionAsteroidPodID]d
    echo "#define POINTS_200_ID              ", [Points200ID]d, "// spyship bonuses"
    echo "#define POINTS_400_ID              ", [Points400ID]d
    echo "#define POINTS_800_ID              ", [Points800ID]d
    echo "#define POINTS_500_ID              ", [Points500ID]d, "// formation bonuses"
    echo "#define POINTS_1000_ID             ", [Points1000ID]d
    echo "#define POINTS_1500_ID             ", [Points1500ID]d
    echo "#define GAMEOVER_ID                ", [GameOverID]d